/* 
 * main.c:
 * This is the code that initializes LWIP and hooks it to uMon's
 * packet interface.  This code is *almost* entirely target
 * independent.  The only code that is cpu/target specific is
 * the code that retrieves the time base.
 */

#include <string.h>
#include <stdlib.h>
#include "monlib.h"
#include "cfg.h"
#include "lwip/mem.h"
#include "lwip/memp.h"
#include "netif/etharp.h"
#include "lwip/init.h"
#include "lwip/ip.h"
#include "lwip/udp.h"
#include "lwip/tcp.h"
#include "lwip/inet.h"
#include "lwip/tcpip.h"
#include "lwip/netif.h"
#include "lwip/stats.h"
#include "httpd.h"
#include "monsrvr.h"
#include "umonlwip.h"

unsigned long AppStack[APPSTACKSIZE/4];

int
main(int argc,char *argv[])
{
	lwip_umon_startup();

	httpd_init();
	monsrvr_init();

	mon_printf("System running (hit any char to terminate)...\n");

	while(!mon_gotachar())
	{
		lwip_poll();

		/* PUT OTHER APPLICATION CODE HERE */
	}
	return(0);
}

void
__gccmain()
{
}

int
Cstart(void)
{
	char	**argv;
	int		argc;

	/* Connect the application to the monitor.  This must be done
	 * prior to the application making any other attempts to use the
	 * "mon_" functions provided by the monitor.
	 */
	monConnect((int(*)())(*(unsigned long *)MONCOMPTR),(void *)0,(void *)0);

	/* When the monitor starts up an application, it stores the argument
	 * list internally.  The call to mon_getargv() retrieves the arg list
	 * for use by this application...
	 */
	mon_getargv(&argc,&argv);

	/* Call main, then exit to monitor.
	 */
	main(argc,argv);

	mon_appexit(0);

	/* Won't get here. */
	return(0);
}

/* CstartAlt():
 * Demonstrates the use of the "call -a" command in uMon. 
 * For example, if for some reason you wanted to do this...
 * Load the application then load the symtbl file using
 * "make TARGET_IP=1.2.3.4 sym", then issue these commands:
 *
 *  tfs -v ld app
 *  call -a %CstartAlt one two three
 *
 * The "call -a" command in uMon correctly sets up the function
 * call parameters so that the following function would see 4
 * arguments (including arg0), with argv[1] thru argv[3] being
 * pointers to each of the number strings (i.e. "one", "two", "three")
 * and argv[0] being the ascii-coded-hex address of the function
 * CstartAlt.
 */
int
CstartAlt(int argc, char *argv[])
{
	monConnect((int(*)())(*(unsigned long *)MONCOMPTR),(void *)0,(void *)0);
	main(argc,argv);
	mon_appexit(0);
	return(0);
}

#if CPU_IS_PPC
clock_time_t
clock_time()
{
    volatile unsigned register tbl;

    // Retrieve the current value of the TBL register:
    asm volatile ("mfspr %0, 0x10c" : "=r" (tbl));

    return((clock_time_t)tbl);
}
#elif CPU_IS_MIPS
#error: Gotta write clock_time for MIPS.
#elif CPU_IS_ARM
#error: Gotta write clock_time for ARM.
#elif CPU_IS_BFIN
/* clock_time is in assembler */
#elif CPU_IS_MICROBLAZE
#error: Gotta write clock_time for MICROBLAZE.
#elif CPU_IS_68K
#else
#error: Invalid CPU specification.
#endif
