//=============================================================================
//
//      etherdev.c
//
//      Ethernet Abstraction Layer for Micro Monitor
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         03-
// Description:  This file contains the interface layer between Micro Monitor
//               and the Ethernet driver for the cs8900a on the CSB226.
//
//=============================================================================

#include "config.h"
#include "cpuio.h"
#include "genlib.h"
#include "stddefs.h"
#include "ether.h"

// include this or something like it if we have the one wire ip installed
//#include <onewire/altera_avalone_onewire_regs.h>

extern void lan91c11x_reset(void);
extern ushort lan91c11x_rx(uchar *pktbuf);
extern int lan91c11x_init(void);
extern ulong lan91c11x_tx(ulong txbuf, ulong length);
extern void lan91c11x_show_stats(void);



uchar tx_buf[1600];

typedef struct
{
  uchar   cLength;
  uchar  acProjId[4];
  uchar  acMacAddr[6];
  uchar  acChkSum[2]; 

} NODE_ADDR, * PNODE_ADDR;

NODE_ADDR NodeAddr;

unsigned char theMac[18];

#if INCLUDE_ETHERNET

/*
 * enreset():
 *  Reset the PHY and MAC.
 */
void
enreset(void)
{
  lan91c11x_reset();
}

/*
 * eninit():
 * This would include establishing buffer descriptor tables and
 * all the support code that will be used by the ethernet device.
 *
 * It can be assumed at this point that the array uchar BinEnetAddr[6]
 * contains the 6-byte MAC address.
 *
 * Return 0 if successful; else -1.
 */
int
eninit(void)
{
  return lan91c11x_init();

}

int
EtherdevStartup(int verbose)
{
  // device error counters are handled in lan91c11x.c

  /* Put ethernet controller in reset: */
  enreset();

  /* Initialize controller: */
  eninit();

  return(0);
}

/* disablePromiscuousReception():
 * Provide the code that disables promiscuous reception.
 */
void
disablePromiscuousReception(void)
{
  /* OPT_ADD_CODE_HERE */  
}

/* enablePromiscuousReception():
 * Provide the code that enables promiscuous reception.
 */
void
enablePromiscuousReception(void)
{
  /* OPT_ADD_CODE_HERE */
}

/* disableBroadcastReception():
 * Provide the code that disables broadcast reception.
 */
void
disableBroadcastReception(void)
{
  /* ADD_CODE_HERE */
}

/* enableBroadcastReception():
 * Provide the code that enables broadcast reception.
 */
void
enableBroadcastReception(void)
{
  /* ADD_CODE_HERE */
}

/* 
 * enselftest():
 *  Run a self test of the ethernet device(s).  This can be stubbed
 *  with a return(1).
 *  Return 1 if success; else -1 if failure.
 */
int
enselftest(int verbose)
{
  return(1);
}

/* ShowEtherdevStats():
 * This function is used to display device-specific stats (error counts
 * usually).
 */
void
ShowEtherdevStats(void)
{
  lan91c11x_show_stats();
}

/* getXmitBuffer():
 * Return a pointer to the buffer that is to be used for transmission of
 * the next packet.  Since the monitor's driver is EXTREMELY basic,
 * there will only be one packet ever being transmitted.  No need to queue
 * up transmit packets.
 */
uchar *
getXmitBuffer(void)
{
  return tx_buf;
}

/* sendBuffer():
 * Send out the packet assumed to be built in the buffer returned by the
 * previous call to getXmitBuffer() above.
 */
int
sendBuffer(int length)
{
  ulong temp32;

#if INCLUDE_ETHERVERBOSE
  if (EtherVerbose &  SHOW_OUTGOING)
    printPkt((struct ether_header *)tx_buf,length,ETHER_OUTGOING);
#endif

  /* Bump up the packet length to a minimum of 64 bytes.
   */
  if (length < 64)
    length = 64;

  // tell the lan91c11x to send the tx buffer pointed to by tx_buf
  temp32 = lan91c11x_tx((ulong)tx_buf, (ulong)length);

  EtherXFRAMECnt++;
  if (temp32) {
    return -1;
  }
  else {
    return 0;
  }
}

/* DisableEtherdev():
 * Fine as it is...
 */
void
DisableEtherdev(void)
{
  enreset();
}

/* extGetIpAdd():
 * If there was some external mechanism (other than just using the
 * IPADD shell variable established in the monrc file) for retrieval of
 * the board's IP address, then do it here...
 */
char *
extGetIpAdd(void)
{
  return((char *)0);
}

/* extGetEtherAdd():
 * If there was some external mechanism (other than just using the
 * ETHERADD shell variable established in the monrc file) for retrieval of
 * the board's MAC address, then do it here...
 */
char *
extGetEtherAdd(void)
{
  /*
  int             iTemp     = 0; // must be an int!!!!
  unsigned char   cTemp     = 0;
  unsigned char * pNodeAddr = (unsigned char *)&NodeAddr;
  unsigned char * pTheMac   = (unsigned char *)&theMac;
  
  // set the clock 
  IOWR_ALTERA_AVALON_ONEWIRE_CLK_DIV( ONEWIRE_0_BASE, ALTERA_AVALON_ONEWIRE_CLK_80MHZ | ALTERA_AVALON_ONEWIRE_CLK_EN_MSK);  
  
  // issue a master reset to the one-wire bus
  IOWR_ALTERA_AVALON_ONEWIRE_CR(      ONEWIRE_0_BASE, ALTERA_AVALON_ONEWIRE_1WR_MSK);
  
  // delay 2 milliseconds ( presence detect occurs within 960us)
  monDelay(2);
  
  // read the interrupt register - if a device is present 
  iTemp = IORD_ALTERA_AVALON_ONEWIRE_INT( ONEWIRE_0_BASE);
  
  if ( iTemp & ALTERA_AVALON_ONEWIRE_PD_MSK)
  {
    // now test for slave
    if ( iTemp & ALTERA_AVALON_ONEWIRE_PDR_MSK)
    {
        printf("On One-Wire Device Not Detected\n");
    }
  }
  
  // we are the only device on the bus - skip the reading of the rom
  IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, ALTERA_AVALON_ONEWIRE_SKIP_ROM);
  monDelay(2);
  
  // write the read memory
  IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, ALTERA_AVALON_ONEWIRE_READ_MEMORY);
  monDelay(2); 

  // we must send a two byte address into the rom - we will start reading at 0000   
  IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, 0);
  monDelay(2);
   
  IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, 0);
  monDelay(2); 
  
  // the first byte is the crc of the read command - read and discard
  IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, 0xff);
  monDelay(2);

  iTemp = IORD_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE);

  // clear the buffer
  memset( (unsigned char *)&NodeAddr, 0, sizeof( NODE_ADDR));

  // read the mac address from the dallas chip    
  for ( iTemp = 0; iTemp < sizeof( NODE_ADDR); iTemp++, pNodeAddr++)
  {
      IOWR_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE, 0xff);
      monDelay(2);

      *pNodeAddr = (char)IORD_ALTERA_AVALON_ONEWIRE_DATA_BUF( ONEWIRE_0_BASE);
  }
  
  memset( theMac, 0, sizeof(theMac));
  
  // test for validity ( company id = 00 60 35)
  if (!(( NodeAddr.acMacAddr[5] == 0x0) && ( NodeAddr.acMacAddr[4] == 0x60) && ( NodeAddr.acMacAddr[3] == 0x35)))
  {
    memset( (unsigned char *)theMac, 0, sizeof( theMac));
  }
  else
  {
    for( iTemp = 5; iTemp >= 0; iTemp--)
    {

        cTemp     = NodeAddr.acMacAddr[ iTemp] >> 4;
       *pTheMac++ = ( cTemp > 9) ? cTemp + 0x57: cTemp + 0x30;
      
       cTemp      = NodeAddr.acMacAddr[ iTemp] & 0x0f;
       *pTheMac++ = ( cTemp > 9) ? cTemp + 0x57: cTemp + 0x30;     
      
      if ( iTemp > 0)
      {
         *pTheMac++ = ':';
      }
    }
  }

  return (char *)theMac;
  */
    return (char *)0;
}

/*
 * polletherdev():
 * Called continuously by the monitor (ethernet.c) to determine if there
 * is any incoming ethernet packets.
 */
int
polletherdev(void)
{
  uchar pktbuf[RBUFSIZE];
  int pktlen = 0, pktcnt = 0;

  pktlen = lan91c11x_rx(pktbuf);

  if(pktlen){
    pktcnt++;
    EtherRFRAMECnt++;
//    processPACKET((struct ether_header *)pktbuf, pktlen);
  }
  return(pktcnt);
}

#endif
