#include "config.h"
#include "phyeeprom.h"

#if PLATFORM_PCORE537

/*#define __DEBUG_NAK__*/

#ifdef __DEBUG_NAK__
typedef struct
{
    unsigned short header;
    unsigned short	location;
    unsigned short masterCtl;
    unsigned short masterStat;
    unsigned short intStat;
    unsigned short fifoCtl;
    unsigned short fifoStat;
} _twiStatii;

_twiStatii twiStatii[0xffff] = {0};
unsigned short statiiCount = 0;

void captureStatii(unsigned short location)
{
 /*   static unsigned short count = 0;*/
    _twiStatii *stat;
    
    if(statiiCount < 0xffff){
   
	    stat = &(twiStatii[statiiCount]);
    
	    stat->header = 0x1234;
	 	stat->location = location;
	 	stat->masterCtl = *pTWI_MASTER_CTL;
	 	stat->masterStat = *pTWI_MASTER_STAT;
	 	stat->intStat = *pTWI_INT_STAT;
	 	stat->fifoCtl = *pTWI_FIFO_CTL;
	 	stat->fifoStat = *pTWI_FIFO_STAT;
    
	 	statiiCount++;
    }
}


#define CAPSTAT(x)	captureStatii(x)

#else

#define CAPSTAT(x)  

#endif



unsigned char eeprom_address = 0x50;

void eeprom_select(unsigned char select)
{
 	eeprom_address = (select & 0x07) | 0x50;   
 	
/*	TWI_MASTER_ADDR.MADDR = eeprom_address;*/
	*pTWI_MASTER_ADDR	= eeprom_address;
 asm("ssync;");
}

void eeprom_init(void)
{	
/*	TWI_CONTROL.PRESCALE = 13;*/ /* PRESCALE = fSCLK/10MHz --> 100/10 ~= 10 */
/*	TWI_CONTROL.TWI_ENA	= 1; */  /* enable TWI interface */

	*pTWI_CONTROL = (PRESCALE & 10) | TWI_ENA;
	
	
/*	TWI_CLKDIV.CLKLOW = 11;*/
/*	TWI_CLKDIV.CLKHI = 11;*/

	*pTWI_CLKDIV = CLKLOW(11) | CLKHI(11);
	
/*	TWI_MASTER_ADDR.MADDR = eeprom_address; */
	*pTWI_MASTER_ADDR = eeprom_address;
 asm("ssync;");
}

/*#define CAPSTAT(x)	x */
/*Send two bytes (usually address) and check for NAK (in case the EEPROM is currently writing)*/
unsigned int errorCounter=0;

void eeprom_sendTwoBytes(unsigned short word, unsigned char numBytesInTransmission)
{
    unsigned short temp;
    unsigned char sendByte = 1;
 /*   unsigned short errorCounter = 0;*/
    
/*	TWI_XMT_DATA8.XMTDATA8 = address >> 8; */		/* upper bits of EEPROM write adx */
/*	TWI_XMT_DATA8.XMTDATA8 = address & 0xff; */	/* lower bits of EEPROM write adx */

/*--DBG_0		Initial */

/*	CAPSTAT(1);         */
	
	do{
	    
	   	/*Check if the device didn't send an ACK (it's still writing)*/
   		if((*pTWI_MASTER_STAT & ANAK)) /*&& (*pTWI_INT_STAT & MERR)) */
		{
		    CAPSTAT(2);
		    
		 	sendByte = 1;
			
		 	while(!(*pTWI_INT_STAT & MCOMP));
		 	
		 	CAPSTAT(3);
		 	*pTWI_INT_STAT = MERR;
		 	*pTWI_MASTER_STAT = ANAK;
		 	*pTWI_INT_STAT = MCOMP;
		 	
 asm("ssync;");
		 			 	
		 	errorCounter++;
		 	CAPSTAT(4);
		 	
		}
		
		if(sendByte)
	    {
	        CAPSTAT(5);
	        *pTWI_XMT_DATA8 = word >> 8;
			*pTWI_XMT_DATA8 = word & 0x00ff; 
		/*--DBG_A		Load two bytes*/


		/*	TWI_MASTER_CTL_REG = TWI_DCNT(3) | TWI_FAST | TWI_MSTR_XMT | TWI_MSTR_ENA;*/

		/*	*pTWI_MASTER_CTL = (DCNT & (3 << 6)) | FAST | MEN; */
			temp = (DCNT & (numBytesInTransmission << 6)) | FAST | MEN;
			*pTWI_MASTER_CTL = temp;

 asm("ssync;");
			
			sendByte = 0;
			CAPSTAT(6);
	    }

		
	}while (!(*pTWI_INT_STAT & XMTSERV)); /*wait for a byte to become available*/
	
	CAPSTAT(7);
/*	TWI_INT_STAT.XMTSERV = 1;*/ /* clear */
	*pTWI_INT_STAT = XMTSERV;
	CAPSTAT(8);
 asm("ssync;");
}


void eeprom_write_byte(unsigned short address, unsigned char data)
{
    CAPSTAT(0x1000);
	eeprom_sendTwoBytes(address, 3);
	
	CAPSTAT(9);
/*	TWI_XMT_DATA8.XMTDATA8 = data;*/ /* data to write at adx location */
	*pTWI_XMT_DATA8 = data & 0xff; 

 asm("ssync;");

	CAPSTAT(0x0A);
/*	while (!TWI_INT_STAT.MCOMP);*/
	while (!(*pTWI_INT_STAT & MCOMP));
	
/*	TWI_INT_STAT.MCOMP = 1;*/
	*pTWI_INT_STAT = MCOMP | XMTSERV;
	
	CAPSTAT(0x0B);
 asm("ssync;");
}

void eeprom_set_address(unsigned short address)
{
    CAPSTAT(0x2000);
	eeprom_sendTwoBytes(address, 2);

	CAPSTAT(0x0C);
/*	while (!TWI_INT_STAT.MCOMP);*/
	while (!(*pTWI_INT_STAT & MCOMP));

/*	TWI_INT_STAT.MCOMP = 1;       */
	*pTWI_INT_STAT = MCOMP | XMTSERV;
	CAPSTAT(0x0D);
 asm("ssync;");
}

unsigned char eeprom_read_byte(void)
{
	unsigned char data;
	CAPSTAT(0x0E);
/*	TWI_MASTER_CTL_REG = TWI_DCNT(1) | TWI_FAST | TWI_MSTR_RCV | TWI_MSTR_ENA;*/
	*pTWI_MASTER_CTL = (DCNT & (1 << 6)) | FAST | MDIR | MEN;
	
	CAPSTAT(0x0F);
 asm("ssync;");
	
/*	while (TWI_FIFO_STAT.RCVSTAT < 1);*/ /* wait for byte to arrive */
	while(!(*pTWI_FIFO_STAT & RCVSTAT));
	
/*	data = (unsigned char)TWI_RCV_DATA8.RCVDATA8;*/
	data = (unsigned char)*pTWI_RCV_DATA8 & 0xff;
 asm("ssync;");
	
	CAPSTAT(0x10);
/*	while (!TWI_INT_STAT.MCOMP);*/
	while (!(*pTWI_INT_STAT & MCOMP));

/*	TWI_INT_STAT.MCOMP = 1;*/
	*pTWI_INT_STAT = MCOMP;
 asm("ssync;");
	CAPSTAT(0x11);

	return data;
}

#if 0  /* Processor is initialised so drop this function */
void eeprom_init_(void)
{

	int temp;
	temp = cli();

	/*Initialize the system clock to 500MHz/125MHz
	  If choosing a different clock speed, it will be necessary 
	  to modify the TWI prescaler (in eeprom_init())*/
   	*pPLL_CTL = (20 << 9) | (*pPLL_CTL & ~MSEL);
	*pPLL_DIV = 4 | (*pPLL_DIV & ~SSEL);
	idle();
	sti(temp);
   	/*    clk_init(20,4);*/ /* 500MHz/125MHz*/
    
	eeprom_init();
}
#endif


void eeprom_getMAC(unsigned char *addressArray)
{
	unsigned char i;
	
    for(i=0; i < 6; i++)
	{
		
	  /*  eeprom_paranoidDelay();*/
	    /*Make sure the register configurations aren't interrupted by preemption*/

	    eeprom_set_address(i);
	    
/*	    eeprom_paranoidDelay();*/
	    addressArray[i] = eeprom_read_byte();
	}
}

/*Add a delay for the heck of it...
Actually, it's not for the heck of it at all, 
it doesn't seem to function without a delay... 
lots of attempts at doing it "the right way" 
by reading transmission complete flags, etc.
but this is the only one that seems to work
0xfffff corresponds to no less than .004s, 
while a three byte transmission takes only
.00006s*/
void eeprom_paranoidDelay(void)
{
	volatile unsigned int i;
	for (i = 0; i < 0xfffff; i++);
}
#endif
