// board specific defines for micro monitor
#define DEFAULT_BAUD_RATE   	38400
#define MON_CPU_CLOCK       	396000000
#define BUS_DIVISOR				3		// this must natch the value set in rom_reset.s
#define MEMCLK					(MON_CPU_CLOCK/BUS_DIVISOR)

#define LOOPS_PER_USEC			15

// These defines allow us to keep pci code endian neutral
#define PCI_ADD8  				0
#define PCI_ADD16 				0
//#define PCI_DBG					

#define BIG_ENDIAN 		// big endian, when desired

// board specific defines needed by sed135x
#define SED1356_CRT				1
#define SED_REG_BASE			0x18000000	// *RCE3
#define SED_MEM_BASE 			0x18200000
#define SED_STEP				1			// 16-bit port on 16-bit boundry

#define SED_REG16(_x_) 			*(vushort *)((ulong)SED_REG_BASE + (((ulong)_x_ * SED_STEP) ^ 0))	// Control/Status Registers, 16-bit mode
#define RD_FB16(_reg_,_val_) 	((_val_) = *((vushort *)(((ulong)SED_MEM_BASE + ((ulong)(_reg_ * SED_STEP) ^ 0)))))
#define WR_FB16(_reg_,_val_) 	(*((vushort *)(((ulong)SED_MEM_BASE + ((ulong)(_reg_ * SED_STEP)  ^ 0)))) = (_val_))
#define H2SED(_x_)				(((((ushort)_x_) & 0xff00) >> 8) | ((((ushort)_x_) & 0x00ff) << 8))
//#define H2SED(_x_)				(_x_)
//#define SED_DBG
#define SED_GET_PHYS_ADD(_reg_) (vulong)(SED_MEM_BASE + ((_reg_ * SED_STEP) ^ 0))

#define SED_DISP_MODE_CRT		1
#define SED_DISP_MODE_LCD		0

// board specific defines needed by ps2
#define PS2_BASE				0x10000000
#define PS2_STEP				4
#define PS2_REG(_x_)  			*(vuchar *)(PS2_BASE + ((_x_ * PS2_STEP) + 3))	// 8-Bit port on 32-bit boundry      
//#define PS2_DEBUG
#define PS2_TIMEOUT				100000

// The Au1500 uses mostly the same bit definitions as the 16552, but
// the register locations are different.
#define SIO_STEP				4
#define UART1_BASE				0xb1400000	// Au1500 UART 3
#define UART2_BASE				0xb1100000	// Au1500 UART 0

#define UART_RXD				0x000	// receive data, read
#define UART_TXD				0x004	// transmit data, write
#define	UART_IEN				0x008	// interrupt enable, read/write
#define	UART_ISTAT				0x00c	// interrupt status, read - different bit defines than 16c552
#define	UART_FCTL				0x010	// fifo control, write
#define	UART_LCTL				0x014	// line control, read/write
#define	UART_MCTL				0x018	// modem control read/write
#define	UART_LSTAT				0x01c	// line status read
#define	UART_MSTAT				0x020	// modem status read
#define UART_CDR				0x028	// clock divisor register
#define UART_ER					0x100	// module enable register

// additional bit defines
#define UART_ER_ME				0x01	// module enable bit
#define UART_ER_CE				0x02	// clock enable, set before ME

// Access macros used by cpuio.c
#define UARTA(_x_)  *(vulong *)((UART1_BASE + _x_) ^ 0)
#define UARTB(_x_)  *(vulong *)((UART2_BASE + _x_) ^ 0)

// I2C bit banging macros required by i2c.c
#define I2C_SCL_CLR				GPIO_out(207)		// puts 0 on pin
#define I2C_SCL_SET				GPIO_in(207)		// releases pin to be pulled high
#define I2C_SDA_CLR				GPIO_out(206)		// puts 0 on pin		
#define I2C_SDA_SET				GPIO_in(206)		// releases pin to be pulled high		
#define I2C_SCL_RD				GPIO_tst(207)
#define I2C_SDA_RD				GPIO_tst(206)
#define I2C_DELAY 				udelay(100)
