//=============================================================================
//
//      hw_init2.c
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         08/18/03
// Description:  This file contains the 2nd stage initialization called
//				 at boot time.  It assumes the stack has been setup to
//			     point to the internal SRAM.
//
//=============================================================================

#include "stddefs.h"
#include "ml67q5003.h"
#include "cpuio.h"

int cpu_type_4k;

void sdram_init(void);
void hw_init2(void);

void hw_init2()
{
	//************************************************************************
	// 2nd phase initialization
	//************************************************************************

	// Before we go any further we need to find out what 
	// CPU we are running on, ML67Q400x or ML67Q500x.  To
	// do this we attempt to set the IO23 width to 16 in the 
	// BCT_BWC register.  As this field only exists in the 4K,
	// the result tells us what we are running on.  And, most 
	// importantly, since register exists in both devices, we 
	// won't get an abort when we try this.  We did this in
	// reset.S, but we need a C flag for the rest of the code
//	cpu_type_4k = 0;
//	BCTL_REG(BCTL_BWC) |= BCTL_BWC_IO23EN16;

//	if (BCTL_REG(BCTL_BWC) & BCTL_BWC_IO23EN16)
//		cpu_type_4k = 1;
		
	// select the desired secondary functions
	MISC_REG(MISC_GPCTL) = MISC_GPCTL_SIO	// we use the SIO for our console
						 | MISC_GPCTL_ADD;	// enable the upper address bits

	MISC_REG(MISC_BCKCTL) = 0;	// all clocks enabled

	// setup the GPIO
	GPIOA_REG(GPIO_PM) = 0;		// Port A all inputs
	GPIOB_REG(GPIO_PM) = 0x3F;	// Port B 0 - 6 are outputs to the LED
	GPIOC_REG(GPIO_PM) = 0x83;	// Port C 0 & 1 and 7 are outputs to the LED
	GPIOD_REG(GPIO_PM) = 0;		// Port D all inputs
	GPIOE_REG(GPIO_PM) = 0;		// Port E all inputs

	GPIOC_REG(GPIO_PO) |= 0x80;	// force smc into reset

	LED_1();

	// set up the RAM, ROM and IO0 & 1
	BCTL_REG(BCTL_BWC) = BCTL_BWC_ROMEN16 
					   | BCTL_BWC_RAMEN16
					   | BCTL_BWC_IO01EN16;

	if (cpu_type_4k){	// 33Mhz = 30ns
		BCTL_REG(BCTL_ROMAC) = BCTL_ROMAC_4K_4;
		BCTL_REG(BCTL_RAMAC) = BCTL_RAMAC_4K_2;
		BCTL_REG(BCTL_IO01AC) = BCTL_IO01AC_4;
	}
	else {				// 60Mhz = 15ns
		BCTL_REG(BCTL_ROMAC) = BCTL_ROMAC_5K_8;
		BCTL_REG(BCTL_RAMAC) = BCTL_RAMAC_5K_5;
		BCTL_REG(BCTL_IO01AC) = BCTL_IO01AC_8;
	}
	
	LED_2();

#if 0
	// SDRAM Initialization
	// do not intialize the SDRAM if it is already running
	// (such as when we are loaded into ram via JTAG)
	if ((DRAM_REG(DRAM_BWC) & 0x03) != DRAM_BWC_EN16)	// are we already enabled?
	{
		sdram_init();
	}
#endif

	LED_3();

	return;
}

void
sdram_init()
{
	volatile int i;

	DRAM_REG(DRAM_RFSH0) = DRAM_RFSH0_X1;
	DRAM_REG(DRAM_RFSH1) = 0x1d6; 

	DRAM_REG(DRAM_BWC) = DRAM_BWC_EN16;		// set the bus width

	DRAM_REG(DRAM_RPC) = DRAM_RPC_SD_2_4_2_2;	// set access parameters

	for(i=0;i<1000;i++){}

	// send 2 nop commands
	DRAM_REG(DRAM_CMD) = DRAM_CMD_SD_NOP;
	DRAM_REG(DRAM_CMD) = DRAM_CMD_SD_NOP;

	for(i=0;i<1000;i++){}

	// send a pre-charge all command
	DRAM_REG(DRAM_CMD) = DRAM_CMD_SD_PALL;

	for(i=0;i<1000;i++){}

    // send 8 refresh commands
    for(i=0; i<8; i++) {
        DRAM_REG(DRAM_CMD) = DRAM_CMD_SD_REF;
	}

	for(i=0;i<1000;i++){}

    // Set the DRAM control register
	DRAM_REG(DRAM_RMC) = DRAM_RMC_COL8	// DRAM has 8 column bits
		| DRAM_RMC_SDRAM		// DRAM is SYNC type
		| DRAM_RMC_PRE_2		// Precharge Latency is 2 clocks
		| DRAM_RMC_DIST_CBR;	// Enable distributed CBR refreshes

	// send the mode register set command
	DRAM_REG(DRAM_SDMD) = DRAM_SDMD_CL2		// set CL2 in SDRAM mode register
		| DRAM_SDMD_MRS;

	for(i=0;i<1000;i++){}

	// set power down delay (only valid if power down is enabled - we don't)
	DRAM_REG(DRAM_RDWC) = DRAM_RDWC_VAL(16);
}
