//==========================================================================
//
// ml67q5003.h
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Contributors: 
// Date:         08/14/2003
// Description:  This file contains register base addresses and offsets
//				 and access macros for the ML67Q5003 on-chip peripherals
//

#include "bits.h"

//---------------------------------------
// Internal SRAM, 4K and 5K
#define INT_SRAM_BASE_5K		0x50000000
#define INT_SRAM_5K(_x_)		*(vulong *)(INT_SRAM_BASE_5K + _x_)

#define INT_SRAM_BASE_4K		0x10000000
#define INT_SRAM_4K(_x_)		*(vulong *)(INT_SRAM_BASE_4K + _x_)

#define INT_SRAM_SIZE			0x8000	// 32K

//---------------------------------------
// INT - Interrupt Controller - some more registers are in
// the external interrupt (EINT) block at 0x7BF00000
// 
#define INT_BASE_ADD			0x78000000
#define INT_REG(_x_)			*(vulong *)(INT_BASE_ADD + _x_)
#define INT_IRQ					0x00	// IRQ register
#define INT_IRQS				0x04	// Software interrupt register
#define INT_FIQ					0x08	// FIQ register
#define INT_FIQRAW				0x0C	// FIQRAW register
#define INT_FIQEN				0x10	// FIQ enable register
#define INT_IRN					0x14	// IRQ number register
#define INT_CIL					0x18	// Current interrupt level register
#define INT_ILC0				0x20	// Interrupt level control register 0
#define INT_ILC1				0x24	// Interrupt level control register 1
#define INT_CILCL				0x28	// Current interrupt level clear register
#define INT_CILE				0x2C	// Current interrupt level encode register

//---------------------------------------
// BCTL - Bus Control
#define BCTL_BASE_ADD			0x78100000
#define BCTL_REG(_x_)			*(vulong *)(BCTL_BASE_ADD + _x_)
#define BCTL_BWC				0x00	// Bus width control register
#define BCTL_ROMAC				0x04	// External ROM access control register
#define BCTL_RAMAC				0x08	// External RAM access control register
#define BCTL_IO01AC				0x0C	// External I/O bank 0/1 access control register
#define BCTL_IO23ACX			0x10	// External I/O bank 2/3 access control register X - ML674001 Series only

// Bit Defines
// BCTL_BWC - Bus width control register
#define BCTL_BWC_ROMEN16		(2 << 2) 	// only 16-Bit is supported, 0 makes it inaccessable
#define BCTL_BWC_RAMEN16		(2 << 4) 	// only 16-Bit is supported, 0 makes it inaccessable
#define BCTL_BWC_IO01EN8		(1 << 6) 	// enable IOCS 0 and 1 at 8-bits
#define BCTL_BWC_IO01EN16		(2 << 6) 	// enable IOCS 0 and 1 at 16-bits
#define BCTL_BWC_IO23EN8		(1 << 8) 	// enable IOCS 2 and 3 at 8-bits, ML67Q4001 series only
#define BCTL_BWC_IO23EN16		(2 << 8) 	// enable IOCS 2 and 3 at 16-bits, ML67Q4001 series only

// BCTL_ROMAC - External ROM access control register
// ML67Q4001 series
#define BCTL_ROMAC_4K_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_ROMAC_4K_2			(1 << 0) 	// OE/WE Pulse Width = 2 Clocks
#define BCTL_ROMAC_4K_3			(2 << 0) 	// OE/WE Pulse Width = 3 Clocks
#define BCTL_ROMAC_4K_4			(3 << 0) 	// OE/WE Pulse Width = 4 Clocks
#define BCTL_ROMAC_4K_8			(7 << 0) 	// OE/WE Pulse Width = 8 Clocks
// ML67Q5001 series
#define BCTL_ROMAC_5K_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_ROMAC_5K_2			(1 << 0) 	// OE/WE Pulse Width = 2 Clocks
#define BCTL_ROMAC_5K_5			(3 << 0) 	// OE/WE Pulse Width = 5 Clocks
#define BCTL_ROMAC_5K_8			(4 << 0) 	// OE/WE Pulse Width = 8 Clocks
#define BCTL_ROMAC_5K_10		(5 << 0) 	// OE/WE Pulse Width = 10 Clocks
#define BCTL_ROMAC_5K_16		(7 << 0) 	// OE/WE Pulse Width = 16 Clocks
#define BCTL_ROMAC_BRST			BIT4 	 	// Page Mode Enable, ML67Q5001 series only

// BCTL_RAMAC - External RAM access control register
// ML67Q4001 series
#define BCTL_RAMAC_4K_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_RAMAC_4K_2			(1 << 0) 	// OE/WE Pulse Width = 2 Clocks
#define BCTL_RAMAC_4K_3			(2 << 0) 	// OE/WE Pulse Width = 3 Clocks
#define BCTL_RAMAC_4K_4			(3 << 0) 	// OE/WE Pulse Width = 4 Clocks
#define BCTL_RAMAC_4K_8			(7 << 0) 	// OE/WE Pulse Width = 8 Clocks
// ML67Q5001 series
#define BCTL_RAMAC_5K_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_RAMAC_5K_2			(1 << 0) 	// OE/WE Pulse Width = 2 Clocks
#define BCTL_RAMAC_5K_5			(3 << 0) 	// OE/WE Pulse Width = 5 Clocks
#define BCTL_RAMAC_5K_8			(4 << 0) 	// OE/WE Pulse Width = 8 Clocks
#define BCTL_RAMAC_5K_10		(5 << 0) 	// OE/WE Pulse Width = 10 Clocks
#define BCTL_RAMAC_5K_16		(7 << 0) 	// OE/WE Pulse Width = 16 Clocks
#define BCTL_RAMAC_BRST			BIT4 	 	// Page Mode Enable, ML67Q5001 series only

// BCTL_IO01AC - External I/O bank 0/1 access control register
#define BCTL_IO01AC_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_IO01AC_4			(1 << 0) 	// OE/WE Pulse Width = 4 Clocks
#define BCTL_IO01AC_8			(3 << 0) 	// OE/WE Pulse Width = 8 Clocks
#define BCTL_IO01AC_12			(4 << 0) 	// OE/WE Pulse Width = 12 Clocks
#define BCTL_IO01AC_16			(5 << 0) 	// OE/WE Pulse Width = 16 Clocks
#define BCTL_IO01AC_24			(7 << 0) 	// OE/WE Pulse Width = 24 Clocks

// BCTL_IO23ACX - External I/O bank 2/3 access control register X - ML674001 Series only
#define BCTL_IO23ACX_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_IO23ACX_4			(1 << 0) 	// OE/WE Pulse Width = 4 Clocks
#define BCTL_IO23ACX_8			(3 << 0) 	// OE/WE Pulse Width = 8 Clocks
#define BCTL_IO23ACX_12			(4 << 0) 	// OE/WE Pulse Width = 12 Clocks
#define BCTL_IO23ACX_16			(5 << 0) 	// OE/WE Pulse Width = 16 Clocks
#define BCTL_IO23ACX_24			(7 << 0) 	// OE/WE Pulse Width = 24 Clocks

//---------------------------------------
// EXT - External I/O Bank Control - ML67Q5001 series only
#define EBCTL_BASE_ADD			0x78300000
#define EBCTL_REG(_x_)			*(vulong *)(EBCTL_BASE_ADD + _x_)
#define EBCTL_IO23BWC			0x00	// External I/O bank 2/3 Bus width control register
#define EBCTL_IO23ACY 			0x0C	// External I/O bank 2/3 access control register Y

// Bit Defines
// EBCTL_IO23BWC - External I/O bank 2/3 Bus width control register
#define EBCTL_IO23BWC_EN8		(1 << 8) 	// enable IOCS 2 and 3 at 8-bits, ML67Q4001 series only
#define EBCTL_IO23BWC_EN16		(2 << 8) 	// enable IOCS 2 and 3 at 16-bits, ML67Q4001 series only

// EBCTL_IO23ACY - External I/O bank 2/3 access control register Y
#define BCTL_IO23ACY_1			(0 << 0) 	// OE/WE Pulse Width = 1 Clock
#define BCTL_IO23ACY_4			(1 << 0) 	// OE/WE Pulse Width = 4 Clocks
#define BCTL_IO23ACY_8			(3 << 0) 	// OE/WE Pulse Width = 8 Clocks
#define BCTL_IO23ACY_12			(4 << 0) 	// OE/WE Pulse Width = 12 Clocks
#define BCTL_IO23ACY_16			(5 << 0) 	// OE/WE Pulse Width = 16 Clocks
#define BCTL_IO23ACY_24			(7 << 0) 	// OE/WE Pulse Width = 24 Clocks

//---------------------------------------
// DRAM - DRAM/SDRAM Controller
#define DRAM_BASE_ADD			0x78180000
#define DRAM_REG(_x_)			*(vulong *)(DRAM_BASE_ADD + _x_)
#define DRAM_BWC				0x00	// bus width control register
#define DRAM_RMC				0x04	// control register
#define DRAM_RPC				0x08	// characteristics control register
#define DRAM_SDMD				0x0C	// SDRAM mode register
#define DRAM_CMD				0x10	// command register
#define DRAM_RFSH0				0x14	// refresh cycle control register
#define DRAM_RDWC				0x18	// power down control register
#define DRAM_RFSH1				0x1C	// refresh cycle control register

// Bit Defines
// DRAM_BWC - bus width control register
#define DRAM_BWC_EN8			(1 << 0)	// Enable DRAM @ 8-Bits Wide (EDO Only)
#define DRAM_BWC_EN16			(2 << 0)	// Enable DRAM @ 16-Bits Wide

// DRAM_RMC - control register
#define DRAM_RMC_COL8			(0 << 0)	// DRAM has 8 column bits <-- 4Mx16
#define DRAM_RMC_COL9			(1 << 0)	// DRAM has 9 column bits <-- 8Mx16
#define DRAM_RMC_COL10			(2 << 0)	// DRAM has 10 column bits <-- 16Mx16
#define DRAM_RMC_SDRAM			0			// DRAM is SYNC type
#define DRAM_RMC_EDO			BIT2		// DRAM is EDO type
#define DRAM_RMC_PRE_2			0			// Precharge Latency is 2 clocks
#define DRAM_RMC_PRE_CAL		BIT4		// Precharge Latency = RAS to CAS Latency
#define DRAM_RMC_PDWN			BIT6		// Enable automatic powerdown of DRAM (SDRAM only)
#define DRAM_RMC_DIST_CBR		BIT7		// Enable distributed CAS before RAS refreshes

// DRAM_RPC - characteristics control register
#define DRAM_RPC_SD_1_2_1_1		(0 << 0)	// tRCD, tRAS, tRP, tDPL <-- Optimal for PC100 @ 33Mhz
#define DRAM_RPC_SD_1_3_1_1		(1 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_2_3_2_1		(2 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_2_4_2_1		(3 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_2_4_2_2		(4 << 0)	// tRCD, tRAS, tRP, tDPL <-- Optimal for PC100 @ 60Mhz
#define DRAM_RPC_SD_2_5_2_1		(5 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_2_5_2_2		(6 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_2_5_3_1		(7 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_3_5_3_2		(8 << 0)	// tRCD, tRAS, tRP, tDPL
#define DRAM_RPC_SD_3_6_3_2		(9 << 0)	// tRCD, tRAS, tRP, tDPL

// DRAM_SDMD - SDRAM mode register
#define DRAM_SDMD_CL2			0			// Cas to Data Latency = 2 clocks
#define DRAM_SDMD_CL3			BIT0		// Cas to Data Latency = 3 clocks
#define DRAM_SDMD_MRS			BIT7		// Perform Mode Register Set Cycle

// DRAM_CMD - EDO/SDRAM command register
#define DRAM_CMD_SD_NOP			0			// NOP Command
#define DRAM_CMD_SD_PALL		4			// Precharge All Command
#define DRAM_CMD_SD_REF			5			// Refresh Command
#define DRAM_CMD_SD_SELF		0			// Self Refresh Enter Command
#define DRAM_CMD_SD_SREX		0			// Self-Refresh Exit Command

// DRAM_RFSH0 - refresh cycle control register
#define DRAM_RFSH0_X2			0			// Refresh = clocks specified in RFSH1 x 2
#define DRAM_RFSH0_X1			BIT0		// Refresh = clocks specified in RFSH1 x 1

// DRAM_RDWC - power down control register
#define DRAM_RDWC_VAL(_x_)			((_x_ & 0xf) << 0)	// Power DRAM after (_x_ + 1) idle cycles

// DRAM_RFSH1 - refresh cycle control register
#define DRAM_RFSH1_VAL(_x_)			((_x_ & 0x7ff) << 0)	// Refresh Clocks = CCLK/(_x_)

//---------------------------------------
// DMA - Direct Memory Access
#define DMA_BASE_ADD			0x7BE00000
#define DMA_REG(_x_)			*(vulong *)(DMA_BASE_ADD + _x_)
#define DMA_MOD					0x000	// DMA mode register
#define DMA_STA					0x004	// DMA status register
#define DMA_INT					0x008	// DMA transfer complete status register
#define DMA_MSK0				0x100	// DMA channel 0 mask register
#define DMA_TMOD0				0x104	// DMA channel 0 mode register
#define DMA_SAD0				0x108	// DMA channel 0 source address register
#define DMA_DAD0				0x10C	// DMA channel 0 destination address register
#define DMA_SIZ0				0x110	// DMA channel 0 count register
#define DMA_CINT0				0x114 	// DMA channel 0 complete status clear register
#define DMA_MSK1				0x200 	// DMA channel 1 mask register
#define DMA_TMOD1				0x204 	// DMA channel 1 mode register
#define DMA_SAD1				0x208 	// DMA channel 1 source address register
#define DMA_DAD1				0x20C 	// DMA channel 1 destination address register
#define DMA_SIZ1				0x210 	// DMA channel 1 count register
#define DMA_CINT1				0x214 	// DMA channel 1 complete status clear register

//---------------------------------------
// EINT - External Interrupt Control - not the same as the Internal Interrupt Controller
// This one handles some of the internal (timers) and all the external IRQ sources.
#define EINT_BASE_ADD			0x7BF00000
#define EINT_REG(_x_)			*(vulong *)(EINT_BASE_ADD + _x_)
#define EINT_CL					0x04 	// IRQ clear register
#define EINT_QA					0x10 	// IRQA register
#define EINT_DM					0x14 	// IRQ detection mode setting register
#define EINT_LC					0x18 	// Interrupt level control register

//---------------------------------------
// ADC - Analog to Digital Controller
#define ADC_BASE_ADD			0xB6001000
#define ADC_REG(_x_)			*(vushort *)(ADC_BASE_ADD + _x_)
#define ADC_CON0				0x00 	// ADC control register 0  R/W 16 0x0000 21-3
#define ADC_CON1				0x04 	// ADC control register 1  R/W 16 0x0000 21-5
#define ADC_CON2				0x08 	// ADC control register 2  R/W 16 0x0003 21-6
#define ADC_INT					0x0C 	// ADC interrupt control register  R/W 16 0x0000 21-7
#define ADC_FINT				0x10 	// ADC forced interrupt register  R/W 16 0x0000 21-9
#define ADC_RR0					0x14 	// ADC result register 0  R/W 16 0x0000 21-10
#define ADC_RR1					0x18 	// ADC result register 1  R/W 16 0x0000 21-10
#define ADC_RR2					0x1C 	// ADC result register 2  R/W 16 0x0000 21-10
#define ADC_RR3					0x20 	// ADC result register 3  R/W 16 0x0000 21-10
						   
//---------------------------------------
// MISC - Miscellaneous Chip Control
#define MISC_BASE_ADD			0xB7000000
#define MISC_REG(_x_)			*(vushort *)(MISC_BASE_ADD + _x_)
#define MISC_GPCTL				0x00 	// Port function select register
#define MISC_BCKCTL				0x04 	// Block clock control register
#define MISC_ROMCEL				0x0C 	// ROM select register
						   
// Bit Defines
// MISC_GPCTL - Port function select register
#define MISC_GPCTL_UART			BIT0	// PIOA 0 - 7 = UART
#define MISC_GPCTL_SIO			BIT1	// PIOB 6 - 7 = Debug SIO
#define MISC_GPCTL_ADD			BIT2	// PIOC 2 - 6 = ADD19-23
#define MISC_GPCTL_DMA0			BIT3	// PIOB 0, 1 & 4 = DMA and Timer Out 0
#define MISC_GPCTL_DMA1			BIT4	// PIOB 2, 3 & 5 = DMA and Timer Out 0
#define MISC_GPCTL_PWM			BIT5	// PIOC 0 & 1 = PWM 0 & 1
#define MISC_GPCTL_WAIT			BIT6	// PIOD 0 = WAIT
#define MISC_GPCTL_WR			BIT7	// PIOC 0 = WR (Read/Write)
#define MISC_GPCTL_SSIO			BIT8	// PIOE 0 - 2 = SSIO
#define MISC_GPCTL_I2C			BIT9	// PIOE 3 & 4 = I2C
#define MISC_GPCTL_XINT0		BIT10	// PIOE 5 = XINT0
#define MISC_GPCTL_XINT1		BIT11	// PIOE 6 = XINT1
#define MISC_GPCTL_XINT2		BIT12	// PIOE 7 = XINT2
#define MISC_GPCTL_XINT3		BIT13	// PIOE 8 = XINT3
#define MISC_GPCTL_FIQ			BIT14	// PIOE 9 = FIQ

// MISC_BCKCTL - Block clock control register
#define MISC_BCKCTL_ADC_DIS		BIT0	// Disable the clock to the ADC Block
#define MISC_BCKCTL_PWM_DIS		BIT1	// Disable the clock to the PWM Block
#define MISC_BCKCTL_TMR0_DIS	BIT2	// Disable the clock to the TIMER 0 Block
#define MISC_BCKCTL_TMR1_DIS	BIT3	// Disable the clock to the TIMER 1 Block
#define MISC_BCKCTL_TMR2_DIS	BIT4	// Disable the clock to the TIMER 2 Block
#define MISC_BCKCTL_TMR3_DIS	BIT5	// Disable the clock to the TIMER 3 Block
#define MISC_BCKCTL_TMR4_DIS	BIT6	// Disable the clock to the TIMER 4 Block
#define MISC_BCKCTL_TMR5_DIS	BIT7	// Disable the clock to the TIMER 5 Block
#define MISC_BCKCTL_DRAM_DIS	BIT8	// Disable the clock to the DRAM Block
#define MISC_BCKCTL_DMA_DIS		BIT9	// Disable the clock to the DMA Block
#define MISC_BCKCTL_UART_DIS	BIT10	// Disable the clock to the UART Block
#define MISC_BCKCTL_SSIO_DIS	BIT11	// Disable the clock to the SSIO Block
#define MISC_BCKCTL_I2C_DIS		BIT12	// Disable the clock to the I2C Block

// MISC_ROMCEL - ROM select register
#define MISC_ROMCEL_RSEL		BIT0	// works with Remap to set the memory at address 0

//---------------------------------------
// I2C - 2-Wire Bus
#define I2C_BASE_ADD			0xB7800000
#define I2C_REG(_x_)			*(vuchar *)(I2C_BASE_ADD + _x_)
#define I2C_CON					0x00 	// I2C bus control register
#define I2C_SAD					0x04 	// I2C bus slave address mode register
#define I2C_CLR					0x08 	// I2C bus transfer speed register
#define I2C_SR					0x0C 	// I2C bus status register
#define I2C_IR					0x10 	// I2C bus interrupt request register
#define I2C_IMR					0x14 	// I2C bus interrupt mask register
#define I2C_DR					0x18 	// I2C bus transmit/receive data register
#define I2C_BC					0x1C 	// I2C bus transfer speed counter
 
//---------------------------------------
// GPIO - General Purpose I/O
#define GPIO_BASE_ADD			0xB7A01000
#define GPIOA_REG(_x_)			*(vushort *)(GPIO_BASE_ADD + 0x00 + _x_)
#define GPIOB_REG(_x_)			*(vushort *)(GPIO_BASE_ADD + 0x20 + _x_)
#define GPIOC_REG(_x_)			*(vushort *)(GPIO_BASE_ADD + 0x40 + _x_)
#define GPIOD_REG(_x_)			*(vushort *)(GPIO_BASE_ADD + 0x60 + _x_)
#define GPIOE_REG(_x_)			*(vushort *)(GPIO_BASE_ADD + 0x80 + _x_)
#define GPIO_PO					0x00 	// GPIO port output register
#define GPIO_PI					0x04 	// GPIO port input register
#define GPIO_PM					0x08 	// GPIO port mode register
#define GPIO_IE					0x0C 	// GPIO interrupt enable register
#define GPIO_IP					0x10 	// GPIO interrupt polarity register
#define GPIO_IS					0x14 	// GPIO interrupt status register

//---------------------------------------
// UART - Asynchronous Serial I/O - bit defines from 16C552.h in csb_common
#define UART_BASE_ADD			0xB7B00000
#define UART_REG(_x_)			*(vuchar *)(UART_BASE_ADD + _x_)
#define UART_DLL				0x00 	// Divisor Latch (LSB) - When DLAB = 1
#define UART_RBR				0x00 	// Receiver Buffer Register
#define UART_THR				0x00 	// Transmitter Holding Register
#define UART_DLM				0x04 	// Divisor Latch (MSB) - When DLAB = 1
#define UART_IER				0x04 	// Interrupt Enable Register
#define UART_FCR				0x08 	// FIFO Control Register
#define UART_IIR				0x08 	// Interrupt Identification Register
#define UART_LCR				0x0C 	// Line Control Register
#define UART_MCR				0x10 	// Modem Control Register
#define UART_LSR				0x14 	// Line Status Register
#define UART_MSR				0x18 	// Modem Status Register
#define UART_SCR				0x1C 	// Scratch Register

//---------------------------------------
// SSIO - Synchronous Serial I/O
#define SSIO_BASE_ADD			0xB7B01000
#define SSIO_REG(_x_)			*(vuchar *)(SSIO_BASE_ADD + _x_)
#define SSIO_BUF				0x00 	// Synchronous SIO transmit-receive buffer register  R/W 8 0x00 19-3
#define SSIO_ST					0x04 	// Synchronous SIO status register  R/W 8 0x00 19-3
#define SSIO_INT				0x08 	// Synchronous SIO interrupt request register  R/W 8 0x00 19-4
#define SSIO_INTEN				0x0C 	// Synchronous SIO interrupt enable register  R/W 8 0x00 19-5
#define SSIO_CON				0x10 	// Synchronous SIO transmit-receive control register  R/W 8 0x00 19-6
#define SSIO_TSCON				0x14 	// Synchronous SIO test control register  R/W 8 0x00 19-7

//---------------------------------------
// PWM - Pulse Width Modulators
#define PWM_BASE_ADD			0xB7D00000
#define PWM_REG(_x_)			*(vushort *)(PWM_BASE_ADD + _x_)
#define PWM_PWR0				0x00 	// PWM register 0
#define PWM_PWCY0				0x04 	// PWM period register 0
#define PWM_PWC0				0x08 	// PWM counter 0
#define PWM_PWCON0				0x0C 	// PWM control register 0
#define PWM_PWR1				0x20 	// PWM register 1
#define PWM_PWCY1				0x24 	// PWM period register 1
#define PWM_PWC1				0x28 	// PWM counter 1
#define PWM_PWCON1				0x2C 	// PWM control register 1
#define PWM_PWINTSTS		0x3C 	// PWM Interrupt status register

//---------------------------------------
// WDOG - Watch Dog Timer
#define WDOG_BASE_ADD			0xB7E00000
#define WDOG_REG(_x_)			*(vuchar *)(WDOG_BASE_ADD + _x_)
#define WDOG_CON				0x00 	// Watchdog timer control register
#define WDOG_TBCON				0x04 	// Time base counter control register
#define WDOG_STAT				0x14 	// Status register

//---------------------------------------
// TMR - Timer
#define TMR_BASE_ADD			0xB7F00000
#define TMR0_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0x00 + _x_)
#define TMR1_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0x20 + _x_)
#define TMR2_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0x40 + _x_)
#define TMR3_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0x60 + _x_)
#define TMR4_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0x80 + _x_)
#define TMR5_REG(_x_)			*(vushort *)(TMR_BASE_ADD + 0xa0 + _x_)
#define TMR_CNTL				0x00 	// Timer control register
#define TMR_BASE				0x04 	// Timer base register
#define TMR_CNT					0x08 	// Timer counter register
#define TMR_CMP					0x0C 	// Timer compare register
#define TMR_STAT				0x10 	// Timer status register

//---------------------------------------
// CLK - Clock Control
#define CLK_BASE_ADD			0xB8000000
#define CLK_REG(_x_)			*(vulong *)(CLK_BASE_ADD + _x_)
#define CLK_STP					0x04 	// Clock stop register
#define CLK_GBCNT0				0x08 	// Clock gear control register
#define CLK_WT					0x0C 	// Clock wait register
#define CLK_RMPCON				0x10 	// Remap control register

// Bit Defines
// CLK_STP - Clock stop register
#define CLK_STP_ENABLE			0x003c	// write this value first before updating register
#define CLK_STP_SIO				BIT0	// Stop the Clock to the Serial I/O Block
#define CLK_STP_TIC				BIT1	// Stop the Clock to the Timer/Counter Block
#define CLK_STP_HALT			BIT2	// Place the Chip in Halt Mode
#define CLK_STP_STBY			BIT7	// Place the Chip in Standby Mode

// CLK_GBCNT0 - Clock gear control register
#define CLK_GBCNT0_ENABLE		0x003c	// write this value first before updating register
#define CLK_GBCNT0_HCLK(_x_)	((_x_ & 0x3) << 0)	// HCLK Divisor from the Base Clock
#define CLK_GBCNT0_CCLK(_x_)	((_x_ & 0x3) << 4)	// CCLK Divisor from the Base Clock

// CLK_WT - Clock wait register
#define CLK_WT_ENABLE			0x003c	// write this value first before updating register
#define CLK_WT_EXTCLK			0		// wait 30ns after standby when using an external clock
#define CLK_WT_CRYSTAL			0xf		// wait 10-24ms after standby when using a crystal

// CLK_RMPCON - Remap control register
#define CLK_RMPCON_VAL(_x_)		((_x_ & 0x0f) << 0)	// remap control bits - see ML67Q5003 data sheet section 3.5

//---------------------------------------
// SYSTM - System Timer
#define SYSTM_BASE_ADD			0xB8001000
#define SYSTM_REG(_x_)			*(vulong *)(SYSTM_BASE_ADD + _x_)
#define SYSTM_EN				0x04 	// System timer enable register
#define SYSTM_RLR				0x08 	// System timer reload register
#define SYSTM_OVFR				0x10 	// System timer overflow register
						   
//---------------------------------------
// SIO - attached directly to core, different than UART
#define SIO_BASE_ADD			0xB8002000
#define SIO_REG(_x_)			*(vulong *)(SIO_BASE_ADD + _x_)
#define SIO_BUF					0x00 	// SIO transfer buffer register
#define SIO_STA					0x04 	// SIO status register
#define SIO_CON					0x08 	// SIO control register
#define SIO_BCN					0x0C 	// Baud rate control register
#define SIO_BT					0x14 	// Baud rate timer register
#define SIO_TCN					0x18 	// SIO test control register

// Bit Defines
// SIO_BUF - SIO transfer buffer register
#define SIO_BUF_VAL(_x_) 		((_x_ & 0xff) << 0)	// transmit/receive buffer

// SIO_STA - SIO status register - write one to bit to clear it
#define SIO_STA_FERR			BIT0	// Framing Error Detected
#define SIO_STA_OERR			BIT1	// Overrun Error Detected
#define SIO_STA_PERR			BIT2	// Parity Error Detected
#define SIO_STA_RXRDY			BIT4	// Receiver has data ready
#define SIO_STA_TXRDY			BIT5	// Transmitter is ready to accept data

// SIO_CON - SIO control register
#define SIO_CON_LN8				0		// Data Length = 8-bits
#define SIO_CON_LN7				BIT0	// Data Length = 7-bits
#define SIO_CON_PEN				BIT1	// Parity Enable
#define SIO_CON_PODD			0		// Odd Parity (if enabled)
#define SIO_CON_PEVEN			BIT2	// Even Parity (if enabled)
#define SIO_CON_STP2			0		// 2 Stop Bits
#define SIO_CON_STP1			BIT3	// 1 Stop Bit

// SIO_BCN - Baud rate control register
#define SIO_BCN_EN				BIT4	// Enable Buad Rate Counter

// SIO_BT - Baud rate timer register, Baud = 256 - (CCLK/(Baud * 16))
#define SIO_BT_VAL(_x_)			((_x_ & 0xff) << 0)

// SIO_TCN - SIO test control register
#define SIO_TCN_MFERR			BIT0	// Make a Framing Error
#define SIO_TCN_MPERR			BIT1	// Make a Parity Error
#define SIO_TCN_LBK				BIT7	// Enable Loopback
