//
// SM501 Video Controller Register and Bit Definitions
//
//---------------------------------------
// The folowing defines should be done in cpuio.h
//#define SM_BASE				0x30000000		// CS2
//#define SM_REG(_x_)  			*(vushort *)(SM_BASE + 0x03e00000 + _x_)	// Register Space accesses
//#define SM_FB(_x_)  			*(vushort *)(SM_BASE + _x_)	// Frame Buffer accesses


//---------------------------------------
//     Register Offsets

// System Configuration Space
// Configuration 1
#define SM_CFG1_CTL			0x00	// System Control
#define SM_CFG1_MISC		0x04	// Miscellaneous Control
#define SM_CFG1_GPIO_LO		0x08	// GPIO31:0 Control
#define SM_CFG1_GPIO_HI		0x0C 	// GPIO63:32 Control
#define SM_CFG1_DRAM		0x10 	// DRAM Control
#define SM_CFG1_ARB			0x14	// Arbitration Control
// Command List
#define SM_CMD_STAT			0x24 	// Command List Status
// Interrupt/Debug
#define SM_INT_RAW			0x28	// Raw Interrupt Read, Clear on Write
#define SM_INT_STAT			0x2c	// Interrupt Status - Read Only
#define SM_INT_MASK			0x30	// Interrupt Mask
#define SM_DBG_CTL			0x34	// Debug Control
// Power Management
#define SM_PWR_CUR_GATE		0x38	// Current Gate
#define SM_PWR_CUR_CLK		0x3c	// Current Clock
#define SM_PWR_MOD0_GATE	0x40	// Power Mode 0 Gate
#define SM_PWR_MOD0_CLK		0x44	// Power Mode 0 Clock
#define SM_PWR_MOD1_GATE	0x48	// Power Mode 1 Gate			   
#define SM_PWR_MOD1_CLK		0x4c	// Power Mode 1 Clock
#define SM_PWR_SLP_GATE		0x50	// Sleep Mode Gate
#define SM_PWR_CTL			0x54	// Power Mode Controler
// Configuration 2
#define SM_CFG2_PCI			0x58	// PCI Master Base Address
#define SM_CFG2_END			0x5c	// Endian Control
#define SM_CFG2_ID			0x60	// Device ID - Read Only
#define SM_CFG2_PLL			0x64	// PLL Clock Count - Read Only
#define SM_CFG2_TMG			0x68	// Miscellaneous Timing
#define SM_CFG2_SDCLK		0x6c	// Current System SDRAM Clock - Read Only



// 16-bit pixels are RGB 565 - LSB of RED and BLUE are tied low at the 
// LCD Interface, while the LSB of GREEN is loaded as 0
#define RED_SUBPIXEL(n)		(n & 0x1f) << 11
#define GREEN_SUBPIXEL(n)	(n & 0x1f) << 6
#define BLUE_SUBPIXEL(n)	(n & 0x1f) << 0

// define a simple VGA style 16-color pallette
#define	LU_BLACK		RED_SUBPIXEL(0x0) | GREEN_SUBPIXEL(0x0) | BLUE_SUBPIXEL(0x0)  
#define	LU_BLUE			RED_SUBPIXEL(0x0) | GREEN_SUBPIXEL(0x0) | BLUE_SUBPIXEL(0xA) 
#define	LU_GREEN		RED_SUBPIXEL(0x0) | GREEN_SUBPIXEL(0xA) | BLUE_SUBPIXEL(0x0) 
#define	LU_CYAN			RED_SUBPIXEL(0x0) | GREEN_SUBPIXEL(0xA) | BLUE_SUBPIXEL(0xA) 
#define	LU_RED			RED_SUBPIXEL(0xA) | GREEN_SUBPIXEL(0x0) | BLUE_SUBPIXEL(0x0) 
#define	LU_VIOLET		RED_SUBPIXEL(0xA) | GREEN_SUBPIXEL(0x0) | BLUE_SUBPIXEL(0xA) 
#define	LU_YELLOW		RED_SUBPIXEL(0xA) | GREEN_SUBPIXEL(0xA) | BLUE_SUBPIXEL(0x0) 
#define	LU_GREY			RED_SUBPIXEL(0xA) | GREEN_SUBPIXEL(0xA) | BLUE_SUBPIXEL(0xA) 
#define	LU_WHITE		RED_SUBPIXEL(0x5) | GREEN_SUBPIXEL(0x5) | BLUE_SUBPIXEL(0x5) 
#define	LU_BRT_BLUE		RED_SUBPIXEL(0x5) | GREEN_SUBPIXEL(0x5) | BLUE_SUBPIXEL(0xF) 
#define	LU_BRT_GREEN	RED_SUBPIXEL(0x5) | GREEN_SUBPIXEL(0xF) | BLUE_SUBPIXEL(0x5) 
#define	LU_BRT_CYAN		RED_SUBPIXEL(0x5) | GREEN_SUBPIXEL(0xF) | BLUE_SUBPIXEL(0xF) 
#define	LU_BRT_RED		RED_SUBPIXEL(0xF) | GREEN_SUBPIXEL(0x5) | BLUE_SUBPIXEL(0x5) 
#define	LU_BRT_VIOLET	RED_SUBPIXEL(0xF) | GREEN_SUBPIXEL(0x5) | BLUE_SUBPIXEL(0xF) 
#define	LU_BRT_YELLOW	RED_SUBPIXEL(0xF) | GREEN_SUBPIXEL(0xF) | BLUE_SUBPIXEL(0x5) 
#define	LU_BRT_WHITE	RED_SUBPIXEL(0xF) | GREEN_SUBPIXEL(0xF) | BLUE_SUBPIXEL(0xF) 

const ushort vga_lookup[] = {
LU_BLACK,		
LU_BLUE,			
LU_GREEN,		
LU_CYAN,			
LU_RED,			
LU_VIOLET,		
LU_YELLOW,		
LU_GREY,			
LU_WHITE,		
LU_BRT_BLUE,		
LU_BRT_GREEN,	
LU_BRT_CYAN,		
LU_BRT_RED,		
LU_BRT_VIOLET,
LU_BRT_YELLOW,
LU_BRT_WHITE	
};

