/*
 *  This source code has been made available to you by IBM on an AS-IS
 *  basis.  Anyone receiving this source is licensed under IBM
 *  copyrights to use it in any way he or she deems fit, including
 *  copying it, modifying it, compiling it, and redistributing it either
 *  with or without modifications.  No license under IBM patents or
 *  patent applications is to be implied by the copyright license.
 *
 *  Any user of this software should understand that IBM cannot provide
 *  technical support for this software and will not be responsible for
 *  any consequences resulting from the use of this software.
 *
 *  Any person who transfers this source code or any derivative work
 *  must include the IBM copyright notice, this paragraph, and the
 *  preceding two paragraphs in the transferred software.
 *
 *  COPYRIGHT   I B M   CORPORATION 1995
 *  LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com     
 * phone:   908-582-2351          
 *
 * This reset code is a combination of code written by several developers.
 * The source code for the IBM "Walnut" BSP was used as a reference for this
 * bootup.
 */

#include "config.h"
#include "arch_ppc.h"
#include "cpu_403.h"
#include "warmstart.h"
#include "cpu.h"

    .extern	MonStack
    .globl  reset, coldstart, warmstart
	.globl	alt_tfsdevtbl_base

	/* Note that etheraddr.S and alttfsdevtbl.S are only defined
     * for boot-build because the ramtst build, these entries are
	 * indirectly referenced through tags in the CSB472_ramtst.ldt
	 * map file template (converted to CSB472_ramtst.ld).
	 */
#ifndef RAMTST_BUILD
#include "etheraddr.S"
#include "alttfsdevtbl.S"
#endif

	.balign 0x100

/* reset:
 * This is the code that is pointed to by the resetvector at 0xfffffffc.
 * If we get here, then we know that we took some kind of "hard" reset.
 */
reset:
coldstart:
    xor     r0,r0,r0            /* Clear R0 */
    addi    r3,r0,INITIALIZE

/* warmstart:
 * This function can be entered either by having previously hit coldstart,
 * or by the monitor source code calling warmstart().  It is assumed to
 * be a function to the C-code, and it is passed a reason for the warmstart.
 * The register which the compiler uses for storage of the calling argument
 * must be saved so that it can be passed to start().
 */
warmstart:
    mr      r15,r3              /* Copy r3 to r15 because between here and */
                                /* the call to start(), r3 is used a lot. */
    xor     r0,r0,r0            /* Clear R0 */
    addi    r4,r0,0x0000        /* Clear and set up some registers.  */
    xor     r0,r0,r0            /* Clear R0 */
    addi    r4,r0,0x0000        /* Clear and set up some registers.  */
    mtsgr   r4
    mtdcwr  r4
    mtevpr  r4
    mtesr   r4                  /* clear Exception Syndrome Reg */
    mtxer   r4                  /* clear Fixed-Point Exception Reg */
    addi    r4,r0,0x1000        /* set ME bit (Machine Exceptions) */
    oris    r4,r4,0x0002        /* set CE bit (Critical Exceptions) */
    mtmsr   r4                  /* change MSR */

    /* Invalidate i-cache, flush d-cache, then shut them both off:
	 * The invalidate/flush is only useful for warmstart so that
	 * memory coherency is maintained prior to shutting down the caches
	 * during startup.
     */
    bl      invalidate_icache
    bl      invalidate_dcache	/* need to change this to flush */
    mtspr	dccr,r0  
	sync
    mtspr	iccr,r0  
	isync

    /* Initialize the Control 0 register
	 * Enable CS4-7 as GPIO
     * Set UART1 for CTS/RTS and set the UART0 and UART1 internal
     * clock and a divisor of 0x11000 (24+1).
     */
    addis   r3,r0,0x00F0        /* Enable CS 1-3 and uart and lwe */
    ori     r3,r3,0x0030    
    mtdcr   cntrl0,r3           /* set CNTRL0 */

    /* Initialize the Control 1 register
	 * Set source of Timer to Internal CPU Clock
	 * Enable the PCI_INT/LWR signal to be LWR
     */
    addis   r3,r0,0x0000        /* Set Timer Source to Internal CPU Clock  */
    ori     r3,r0,0x4000        /* set PCI_INTA/*LWE to *LWE */
    mtdcr   cntrl1,r3         	/* set CNTRL1 */

	/* Configure GPIO led bits.
	 */
    addis   r5,r0,0xef60        /* Base of GPIO Registers  */
    ori     r5,r5,0x0700
	stw		r0,0x00(r5)			/* clear output bits */
	stw		r0,0x18(r5)			/* clear open drain bits */
	stw		r0,0x04(r5)			/* all LEDS off initially */

	bl		led0_on

    /* Initialize External Bus Controller 
	 */
    bl      ext_busctl_init

    /* Initialize SDRAM Controller.
	 * Note: This file is used for both boot and ramtst builds so when we
	 * are running in ramtst build, we don't want to include the SDRAM
	 * initialization...
     */
#ifndef RAMTST_BUILD
    bl      sdram_init
#endif

    /* Clear reservation bit. */
    addis   r4,r0,0x0000
    lwarx   r3,r0,r4            /* get some data/set resv bit */
    stwcx.  r3,r0,r4            /* store out and clear resv bit */

GoToC:
    lis     sp, (MonStack+(MONSTACKSIZE+64))@h
    ori     sp, sp, (MonStack+(MONSTACKSIZE+64))@l
	bl		stkinit

    lis     sp, (MonStack+(MONSTACKSIZE-4))@h
    ori     sp, sp, (MonStack+(MONSTACKSIZE-4))@l
    addi    r7, r0, -8
    and     sp, sp, r7		/* 8-byte aligned (EABI spec) */
    mr      r3,r15			/* Copy r15 to r3. */
	ba		start
	nop

/****************************************************************************
 *
 * Function:     sdram_init
 * Description:  Configures SDRAM memory banks.
 *
 */
    .text
    .align 2
    .globl  sdram_init

sdram_init:
    /* Set MB0CF for bank 0.  2 16-bit wide DRAM devices (16MByte each)
	 * spanning across the 32-bit bus..
     */
    addi    r4,0,mem_mb0cf
    mtdcr   memcfga,r4
#ifdef DRAM_IS_32M
    addis   r4,0,0x0006		/* Base Address = 0, Bank Size = 32MB */
    ori     r4,r4,0x2001	/* Mode 2, Bank Enable On */
#else
    addis   r4,0,0x0000		/* Base address = 0, bank size = 4M */
    ori     r4,r4,0x6001	/* Mode 4, Bank enable on */
#endif
    mtdcr   memcfgd,r4

    /*
     * Delay to ensure 200usec have elapsed since reset. Assume worst
     * case that the core is running 200Mhz:
     *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
     */
    addis   r3,0,0x0000
    ori     r3,r3,0xA000        /* ensure 200usec have passed since reset */
    mtctr   r3
..spinlp2:
    bdnz    ..spinlp2           /* spin loop */

    /*
     * Set memory controller options reg, MCOPT1.
     * Set DC_EN to '1' and BRD_PRF to '01' for 16 byte PLB Burst 
     * read/prefetch.
     */
    addi    r4,0,mem_mcopt1
    mtdcr   memcfga,r4
    addis   r4,0,0x8080         /* set DC_EN=1 */
    ori     r4,r4,0000
    mtdcr   memcfgd,r4

    /*
     * Delay to ensure 10msec have elapsed since reset. This is
     * required for the MPC952 to stabalize. Assume worst
     * case that the core is running 200Mhz:
     *   200,000,000 (cycles/sec) X .010 (sec) = 0x1E8480 cycles
     * This delay should occur before accessing SDRAM.
     */
    addis   r3,0,0x001E
    ori     r3,r3,0x8480        /* ensure 10msec have passed since reset */
    mtctr   r3
..spinlp3:
    bdnz    ..spinlp3           /* spin loop */
    blr

/****************************************************************************
 *
 * Function:     ext_busctl_init
 * Description:  Initializes the External Bus Controller as follows:
 * Bank 0 - Flash 
 */
    .text
    .align 2
    .globl  ext_busctl_init

/*
 * External Bus Configuration Register
 */
ext_busctl_init:
	addi 	r4, 0,ebc0cfg
    mtdcr   ebccfga,r4
    addis   r4,0,0xd84c
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

/*
 * Memory bank 0...
 * The CSB472 may have an AMD29LV160 (1Mx16=2Mbytes)
 * or an INTEL28F640 (4Mx16=8Mbytes)...
 */
#ifdef FLASH_IS_AMD
    addi    r4,0,pb0ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x0185
    ori     r4,r4,0x5200
    mtdcr   ebccfgd,r4

    addi    r4,0,pb0cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xffe3		/* Base = 0xffe00000 */
    ori     r4,r4,0xa000
    mtdcr   ebccfgd,r4
#else	/* Flash is INTEL */
    addi    r4,0,pb0ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x0305
    ori     r4,r4,0x5800
    mtdcr   ebccfgd,r4

    addi    r4,0,pb0cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xff87		/* Base = 0xff800000 */
    ori     r4,r4,0xa000
    mtdcr   ebccfgd,r4
#endif

/*
 * Memory Bank 3, Xilinx CPLD
 * Xilinx control structure...
 *   32-bits of programmable IO:
 *    - offset 0: 32-bit input data register.
 *    - offset 4: 32-bit tri-state control (1 == output).
 *    - offset 8: 32-bit output data register.
 *    - offset c: 4-bit (upper-most nibble) DIP switch input.
 */
    addi    r4,0,pb3ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x0786
    ori     r4,r4,0x9200
    mtdcr   ebccfgd,r4

    addi    r4,0,pb3cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xff01		/* Base = 0xff000000 */
    ori     r4,r4,0xc000
    mtdcr   ebccfgd,r4

    blr

/****************************************************************************
 *
 * Function: invalidate_icache
 */
    .text
    .align 2
    .globl  invalidate_icache

invalidate_icache:
    iccci   r0,r0           /* for 405, iccci invalidates the */ 
    blr                     /* entire I cache  */

/****************************************************************************
 *
 * Function: invalidate_dcache
 */
    .text
    .align 2
    .globl  invalidate_dcache    

invalidate_dcache:    
    addi    r5,0,0x0000     /* clear GPR 6 */
    addi    r7,r0, 256      /* do loop for # of dcache lines */
                            /* NOTE: dccci invalidates both  */
    mtctr   r7              /* ways in the D cache.  Loop 256 */
							/* lines to handle 450GPr as well */
..dcloop:
    dccci   0,r5            /* invalidate line */
    addi    r5,r5, 32       /* bump to next line */
    bdnz    ..dcloop
    blr

/****************************************************************************
 *
 * Function:     ppcDflush
 * Description:  Flush the data cache. This function does NOT turn off the data
 *               cache after the flush. This function makes use of memory
 *               that hopefully should never be in the data cache under normal
 *               circumstances.
 *                       0xFFFFE000-0xFFFFFFFF
 * Input:        none
 * Output:       none
 */
    .text
    .align  2
    .globl  ppcDflush

ppcDflush:
    addis   r9,r0,0x0002        /* set mask for EE and CE msr bits */
    ori     r9,r9,0x8000
    mfmsr   r12                 /* save msr */
    andc    r9,r12,r9
    mtmsr   r9                  /* disable EE and CE */
    addi    r10,r0,0x0001       /* enable data cache for unused memory */
    mfdccr  r9                  /* region 0xF8000000-0xFFFFFFFF via */
    or      r10,r10,r9          /* bit 31 in dccr */
    mtdccr  r10
    addi    r10,r0,128          /* do loop for # of lines */
    addi    r11,r0,4096         /* D cache set size=4K */
    mtctr   r10
    ori     r10,r0,0xE000       /* start at 0xFFFFE000 */
    add     r11,r10,r11         /* add to get to other side of cache line */
..ppcDflush_loop:
    lwz     r3,0(r10)           /* least recently used side */
    lwz     r3,0(r11)           /* the other side */
    dccci   r0,r11              /* invalidate both sides */
    addi    r10,r10,0x0020      /* bump to next line (32 bytes) */
    addi    r11,r11,0x0020      /* bump to next line (32 bytes) */
    bdnz    ..ppcDflush_loop
    sync                        /* allow memory access to complete */
    mtdccr  r9                  /* restore dccr */
    mtmsr   r12                 /* restore msr */
    blr



/* Led functions:
 */
    .globl  led0_on, led1_on, leds_on, leds_off

led0_on:
    addis   r5,r0,0xef60        /* Base of GPIO Registers  */
    ori     r5,r5,0x0700
	addis	r3,r0,0x0002		/* turn on LED0 and 1 tri-state driver */
	stw		r3,0x04(r5)
	blr

led1_on:
    addis   r5,r0,0xef60        /* Base of GPIO Registers  */
    ori     r5,r5,0x0700
	addis	r3,r0,0x0004		/* turn on LED1 and 1 tri-state driver */
	stw		r3,0x04(r5)
	blr

leds_on:
    addis   r5,r0,0xef60   
    ori     r5,r5,0x0700
	addis	r3,r0,0x0006
	stw		r3,0x04(r5)
	blr
	
leds_off:
    addis   r5,r0,0xef60
    ori     r5,r5,0x0700
	stw		r0,0x04(r5)
	blr

