// board specific defines for micro monitor
#define DEFAULT_BAUD_RATE   	38400
#define MON_CPU_CLOCK       	396000000
#define BUS_DIVISOR				3		// this must match the value set in rom_reset.s
#define MEMCLK					(MON_CPU_CLOCK/BUS_DIVISOR)

#define LOOPS_PER_USEC			15

#define BIG_ENDIAN 		// big endian, when desired

// LCD control CPLD
#define CPLD_BASE				0x10000000	// uses TLB lookup - setup in cpuio.c

// The Au1100 uses mostly the same bit definitions as the 16552, but
// the register locations are different.
#define SIO_STEP				4
#define UART0_BASE				0xb1100000	// Au1100 UART 0
#define UART1_BASE				0xb1200000	// Au1100 UART 1

#define UART_RXD				0x000	// receive data, read
#define UART_TXD				0x004	// transmit data, write
#define	UART_IEN				0x008	// interrupt enable, read/write
#define	UART_ISTAT				0x00c	// interrupt status, read - different bit defines than 16c552
#define	UART_FCTL				0x010	// fifo control, write
#define	UART_LCTL				0x014	// line control, read/write
#define	UART_MCTL				0x018	// modem control read/write
#define	UART_LSTAT				0x01c	// line status read
#define	UART_MSTAT				0x020	// modem status read
#define UART_CDR				0x028	// clock divisor register
#define UART_ER					0x100	// module enable register

// additional bit defines
#define UART_ER_ME				0x01	// module enable bit
#define UART_ER_CE				0x02	// clock enable, set before ME

// Access macros used by cpuio.c
#define UARTA(_x_)  *(vulong *)((UART0_BASE + _x_) ^ 0)
#define UARTB(_x_)  *(vulong *)((UART1_BASE + _x_) ^ 0)

// I2C bit banging macros required by i2c.c
#define I2C_SCL_CLR				GPIO_out(GPIO_SCL)		// puts 0 on pin
#define I2C_SCL_SET				GPIO_in(GPIO_SCL)		// releases pin to be pulled high
#define I2C_SDA_CLR				GPIO_out(GPIO_SDA)		// puts 0 on pin		
#define I2C_SDA_SET				GPIO_in(GPIO_SDA)		// releases pin to be pulled high		
#define I2C_SCL_RD				GPIO_tst(GPIO_SCL)
#define I2C_SDA_RD				GPIO_tst(GPIO_SDA)
#define I2C_DELAY 				udelay(100)
