//==========================================================================
//
// pxa270.h
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Contributors: 
// Date:         04/17/2002
// Description:  This file contains register base addresses and offsets
//				 and access macros for the PXA250 on-chip peripherals
//
// Modified by Luis Torrico, Cogent Computer Systems, Inc.
//	03.28.2007 : added all PXA270 register offsets.

#include "bits.h"

//---------------------------------------
// Memory Controller - offsets and bit 
// defines are in pxa270_msc.h
#define MSC_BASE			0x48000000
#define MSC_REG(_x_)		*(vulong *)(MSC_BASE + _x_)	

//---------------------------------------
// LCD Controller
#define LCD_BASE			0x44000000
#define LCD_REG(_x_)		*(vulong *)(LCD_BASE + _x_)	

//---------------------------------------
// DMA Controller
#define DMA_BASE			0x40000000
#define DMA_REG(_x_)		*(vulong *)(DMA_BASE + _x_)	
#define DMA_DCSR0    		0x000		// DMA Control / Status Register for Channel 0  		
#define DMA_DCSR1    		0x004		// DMA Control / Status Register for Channel 1  		
#define DMA_DCSR2    		0x008		// DMA Control / Status Register for Channel 2  		
#define DMA_DCSR3    		0x00c		// DMA Control / Status Register for Channel 3  		
#define DMA_DCSR4    		0x010		// DMA Control / Status Register for Channel 4  		
#define DMA_DCSR5    		0x014		// DMA Control / Status Register for Channel 5  		
#define DMA_DCSR6    		0x018		// DMA Control / Status Register for Channel 6  		
#define DMA_DCSR7    		0x01c		// DMA Control / Status Register for Channel 7  		
#define DMA_DCSR8    		0x020		// DMA Control / Status Register for Channel 8  		
#define DMA_DCSR9    		0x024		// DMA Control / Status Register for Channel 9  		
#define DMA_DCSR10   		0x028		// DMA Control / Status Register for Channel 10  		
#define DMA_DCSR11   		0x02c		// DMA Control / Status Register for Channel 11  		
#define DMA_DCSR12   		0x030		// DMA Control / Status Register for Channel 12  		
#define DMA_DCSR13   		0x034		// DMA Control / Status Register for Channel 13  		
#define DMA_DCSR14   		0x038		// DMA Control / Status Register for Channel 14  		
#define DMA_DCSR15   		0x03c		// DMA Control / Status Register for Channel 15  		
#define DMA_DCSR16    		0x040		// DMA Control / Status Register for Channel 16  		
#define DMA_DCSR17    		0x044		// DMA Control / Status Register for Channel 17  		
#define DMA_DCSR18    		0x048		// DMA Control / Status Register for Channel 18  		
#define DMA_DCSR19    		0x04c		// DMA Control / Status Register for Channel 19  		
#define DMA_DCSR20    		0x050		// DMA Control / Status Register for Channel 20  		
#define DMA_DCSR21    		0x054		// DMA Control / Status Register for Channel 21  		
#define DMA_DCSR22    		0x058		// DMA Control / Status Register for Channel 22  		
#define DMA_DCSR23    		0x05c		// DMA Control / Status Register for Channel 23  		
#define DMA_DCSR24    		0x060		// DMA Control / Status Register for Channel 24  		
#define DMA_DCSR25    		0x064		// DMA Control / Status Register for Channel 25  		
#define DMA_DCSR26   		0x068		// DMA Control / Status Register for Channel 26  		
#define DMA_DCSR27   		0x06c		// DMA Control / Status Register for Channel 27  		
#define DMA_DCSR28   		0x070		// DMA Control / Status Register for Channel 28  		
#define DMA_DCSR29   		0x074		// DMA Control / Status Register for Channel 29  		
#define DMA_DCSR30   		0x078		// DMA Control / Status Register for Channel 30  		
#define DMA_DCSR31   		0x07c		// DMA Control / Status Register for Channel 31  		
#define DMA_DALGN     		0x0a0		// DMA Alignment Register  		
#define DMA_DPCSR     		0x0a4		// DMA Programmed I/O Control Status Register  		
#define DMA_DRQSR0     		0x0e0		// DMA DREQ 0 Status Register  		
#define DMA_DRQSR1     		0x0e4		// DMA DREQ 1 Status Register  		
#define DMA_DRQSR2     		0x0e8		// DMA DREQ 2 Status Register  		
#define DMA_DRQSR3     		0x0ec		// Reserved  		
#define DMA_DINT     		0x0f0		// DMA Interrupt Register  		
#define DMA_DRCMR0   		0x100		// Request to Channel Map Register for DREQ 0 (companion chip request 0)  		
#define DMA_DRCMR1   		0x104		// Request to Channel Map Register for DREQ 1 (companion chip request 1)  		
#define DMA_DRCMR2   		0x108		// Request to Channel Map Register for I2S receive Request  		
#define DMA_DRCMR3   		0x10c		// Request to Channel Map Register for I2S transmit Request  		
#define DMA_DRCMR4   		0x110		// Request to Channel Map Register for BTUART receive Request  		
#define DMA_DRCMR5   		0x114		// Request to Channel Map Register for BTUART transmit Request.  		
#define DMA_DRCMR6   		0x118		// Request to Channel Map Register for FFUART receive Request  		
#define DMA_DRCMR7   		0x11c		// Request to Channel Map Register for FFUART transmit Request  		
#define DMA_DRCMR8   		0x120		// Request to Channel Map Register for AC97 microphone Request  		
#define DMA_DRCMR9   		0x124		// Request to Channel Map Register for AC97 modem receive Request  		
#define DMA_DRCMR10  		0x128		// Request to Channel Map Register for AC97 modem transmit Request  		
#define DMA_DRCMR11  		0x12c		// Request to Channel Map Register for AC97 audio receive Request  		
#define DMA_DRCMR12  		0x130		// Request to Channel Map Register for AC97 audio transmit Request  		
#define DMA_DRCMR13  		0x134		// Request to Channel Map Register for SSP1 receive Request  		
#define DMA_DRCMR14  		0x138		// Request to Channel Map Register for SSP1 transmit Request  		
#define DMA_DRCMR15  		0x13c		// Request to Channel Map Register for SSP2 receive Request  		
#define DMA_DRCMR16  		0x140		// Request to Channel Map Register for SSP2 transmit Request  		
#define DMA_DRCMR17  		0x144		// Request to Channel Map Register for ICP receive Request  		
#define DMA_DRCMR18  		0x148		// Request to Channel Map Register for ICP transmit Request  		
#define DMA_DRCMR19  		0x14c		// Request to Channel Map Register for STUART receive Request  		
#define DMA_DRCMR20  		0x150		// Request to Channel Map Register for STUART transmit Request  		
#define DMA_DRCMR21  		0x154		// Request to Channel Map Register for MMC receive Request  		
#define DMA_DRCMR22  		0x158		// Request to Channel Map Register for MMC transmit Request  		
#define DMA_DRCMR23  		0x15c		// RESERVED
#define DMA_DRCMR24  		0x160		// Request to Channel Map Register for USB endpoint 0 Request
#define DMA_DRCMR25  		0x164		// Request to Channel Map Register for USB endpoint A Request
#define DMA_DRCMR26  		0x168		// Request to Channel Map Register for USB endpoint B Request
#define DMA_DRCMR27  		0x16c		// Request to Channel Map Register for USB endpoint C Request
#define DMA_DRCMR28  		0x170		// Request to Channel Map Register for USB endpoint D Request
#define DMA_DRCMR29  		0x174		// Request to Channel Map Register for USB endpoint E Request
#define DMA_DRCMR30  		0x178		// Request to Channel Map Register for USB endpoint F Request
#define DMA_DRCMR31  		0x17c		// Request to Channel Map Register for USB endpoint G Request
#define DMA_DRCMR32  		0x180		// Request to Channel Map Register for USB endpoint H Request
#define DMA_DRCMR33  		0x184		// Request to Channel Map Register for USB endpoint I Request
#define DMA_DRCMR34  		0x188		// Request to Channel Map Register for USB endpoint J Request
#define DMA_DRCMR35  		0x18c		// Request to Channel Map Register for USB endpoint K Request
#define DMA_DRCMR36  		0x190		// Request to Channel Map Register for USB endpoint L Request
#define DMA_DRCMR37  		0x194		// Request to Channel Map Register for USB endpoint M Request
#define DMA_DRCMR38  		0x198		// Request to Channel Map Register for USB endpoint N Request
#define DMA_DRCMR39  		0x19c		// Request to Channel Map Register for USB endpoint P Request
#define DMA_DRCMR40  		0x1a0		// Request to Channel Map Register for USB endpoint Q Request  		
#define DMA_DRCMR41  		0x1a4		// Request to Channel Map Register for USB endpoint R Request  		
#define DMA_DRCMR42  		0x1a8		// Request to Channel Map Register for USB endpoint S Request  		
#define DMA_DRCMR43  		0x1ac		// Request to Channel Map Register for USB endpoint T Request  		
#define DMA_DRCMR44  		0x1b0		// Request to Channel Map Register for USB endpoint U Request  		
#define DMA_DRCMR45  		0x1b4		// Request to Channel Map Register for USB endpoint V Request  		
#define DMA_DRCMR46  		0x1b8		// Request to Channel Map Register for USB endpoint W Request 
#define DMA_DRCMR47  		0x1bc		// Request to Channel Map Register for USB endpoint X Request  		
#define DMA_DRCMR48  		0x1c0		// Request to Channel Map Register for MSL receive Request 1  		
#define DMA_DRCMR49  		0x1c4		// Request to Channel Map Register for MSL transmit Request 1  		
#define DMA_DRCMR50  		0x1c8		// Request to Channel Map Register for MSL receive Request 2  		
#define DMA_DRCMR51  		0x1cc		// Request to Channel Map Register for MSL transmit Request 2  		
#define DMA_DRCMR52  		0x1d0		// Request to Channel Map Register for MSL receive Request 3   		
#define DMA_DRCMR53  		0x1d4		// Request to Channel Map Register for MSL transmit Request 3
#define DMA_DRCMR54  		0x1d8		// Request to Channel Map Register for MSL receive Request 4 
#define DMA_DRCMR55  		0x1dc		// Request to Channel Map Register for MSL transmit Request 4
#define DMA_DRCMR56  		0x1e0		// Request to Channel Map Register for MSL receive Request 5 
#define DMA_DRCMR57  		0x1e4		// Request to Channel Map Register for MSL transmit Request 5
#define DMA_DRCMR58  		0x1e8		// Request to Channel Map Register for MSL receive Request 6 
#define DMA_DRCMR59  		0x1ec		// Request to Channel Map Register for MSL transmit Request 6
#define DMA_DRCMR60  		0x1f0		// Request to Channel Map Register for MSL receive Request 7 
#define DMA_DRCMR61  		0x1f4		// Request to Channel Map Register for MSL transmit Request 7
#define DMA_DRCMR62  		0x1f8		// Request to Channel Map Register for USIM receive Request
#define DMA_DRCMR63  		0x1fc		// Request to Channel Map Register for USIM transmit Request
#define DMA_DDADR0   		0x200		// DMA Descriptor Address Register channel 0
#define DMA_DSADR0   		0x204		// DMA Source Address Register channel 0
#define DMA_DTADR0   		0x208		// DMA Target Address Register channel 0
#define DMA_DCMD0    		0x20c		// DMA Command Address Register channel 0
#define DMA_DDADR1   		0x210		// DMA Descriptor Address Register channel 1
#define DMA_DSADR1   		0x214		// DMA Source Address Register channel 1
#define DMA_DTADR1   		0x218		// DMA Target Address Register channel 1
#define DMA_DCMD1    		0x21c		// DMA Command Address Register channel 1
#define DMA_DDADR2   		0x220		// DMA Descriptor Address Register channel 2
#define DMA_DSADR2   		0x224		// DMA Source Address Register channel 2
#define DMA_DTADR2   		0x228		// DMA Target Address Register channel 2
#define DMA_DCMD2    		0x22c		// DMA Command Address Register channel 2
#define DMA_DDADR3   		0x230		// DMA Descriptor Address Register channel 3
#define DMA_DSADR3   		0x234		// DMA Source Address Register channel 3
#define DMA_DTADR3   		0x238		// DMA Target Address Register channel 3
#define DMA_DCMD3    		0x23c		// DMA Command Address Register channel 3
#define DMA_DDADR4   		0x240		// DMA Descriptor Address Register channel 4
#define DMA_DSADR4   		0x244		// DMA Source Address Register channel 4
#define DMA_DTADR4   		0x248		// DMA Target Address Register channel 4
#define DMA_DCMD4    		0x24c		// DMA Command Address Register channel 4
#define DMA_DDADR5   		0x250		// DMA Descriptor Address Register channel 5
#define DMA_DSADR5   		0x254		// DMA Source Address Register channel 5
#define DMA_DTADR5   		0x258		// DMA Target Address Register channel 5
#define DMA_DCMD5    		0x25c		// DMA Command Address Register channel 5
#define DMA_DDADR6   		0x260		// DMA Descriptor Address Register channel 6
#define DMA_DSADR6   		0x264		// DMA Source Address Register channel 6
#define DMA_DTADR6   		0x268		// DMA Target Address Register channel 6
#define DMA_CMD6     		0x26c		// DMA Command Address Register channel 6
#define DMA_DADR7    		0x270		// DMA Descriptor Address Register channel 7
#define DMA_SADR7    		0x274		// DMA Source Address Register channel 7
#define DMA_TADR7    		0x278		// DMA Target Address Register channel 7
#define DMA_CMD7     		0x27c		// DMA Command Address Register channel 7
#define DMA_DADR8    		0x280		// DMA Descriptor Address Register channel 8
#define DMA_SADR8    		0x284		// DMA Source Address Register channel 8
#define DMA_TADR8    		0x288		// DMA Target Address Register channel 8
#define DMA_CMD8     		0x28c		// DMA Command Address Register channel 8
#define DMA_DADR9    		0x290		// DMA Descriptor Address Register channel 9
#define DMA_SADR9    		0x294		// DMA Source Address Register channel 9
#define DMA_TADR9    		0x298		// DMA Target Address Register channel 9
#define DMA_CMD9     		0x29c		// DMA Command Address Register channel 9
#define DMA_DADR10   		0x2a0		// DMA Descriptor Address Register channel 10
#define DMA_SADR10   		0x2a4		// DMA Source Address Register channel 10
#define DMA_TADR10   		0x2a8		// DMA Target Address Register channel 10
#define DMA_CMD10    		0x2ac		// DMA Command Address Register channel 10
#define DMA_DADR11   		0x2b0		// DMA Descriptor Address Register channel 11
#define DMA_SADR11   		0x2b4		// DMA Source Address Register channel 11
#define DMA_TADR11   		0x2b8		// DMA Target Address Register channel 11
#define DMA_CMD11    		0x2bc		// DMA Command Address Register channel 11
#define DMA_DADR12   		0x2c0		// DMA Descriptor Address Register channel 12
#define DMA_SADR12   		0x2c4		// DMA Source Address Register channel 12     
#define DMA_TADR12   		0x2c8		// DMA Target Address Register channel 12     
#define DMA_CMD12    		0x2cc		// DMA Command Address Register channel 12     
#define DMA_DADR13   		0x2d0		// DMA Descriptor Address Register channel 13     
#define DMA_SADR13   		0x2d4		// DMA Source Address Register channel 13     
#define DMA_TADR13   		0x2d8		// DMA Target Address Register channel 13     
#define DMA_CMD13    		0x2dc		// DMA Command Address Register channel 13     
#define DMA_DADR14   		0x2e0		// DMA Descriptor Address Register channel 14     
#define DMA_SADR14   		0x2e4		// DMA Source Address Register channel 14     
#define DMA_TADR14   		0x2e8		// DMA Target Address Register channel 14     
#define DMA_CMD14    		0x2ec		// DMA Command Address Register channel 14     
#define DMA_DADR15   		0x2f0		// DMA Descriptor Address Register channel 15     
#define DMA_SADR15   		0x2f4		// DMA Source Address Register channel 15     
#define DMA_TADR15   		0x2f8		// DMA Target Address Register channel 15     
#define DMA_CMD15    		0x2fc		// DMA Command Address Register channel 15     
#define DMA_DADR16   		0x300		// DMA Descriptor Address Register channel 16     
#define DMA_SADR16   		0x304		// DMA Source Address Register channel 16     
#define DMA_TADR16   		0x308		// DMA Target Address Register channel 16     
#define DMA_CMD16    		0x30c		// DMA Command Address Register channel 16     
#define DMA_DADR17   		0x310		// DMA Descriptor Address Register channel 17     
#define DMA_SADR17   		0x314		// DMA Source Address Register channel 17     
#define DMA_TADR17   		0x318		// DMA Target Address Register channel 17     
#define DMA_CMD17    		0x31c		// DMA Command Address Register channel 17     
#define DMA_DADR18   		0x320		// DMA Descriptor Address Register channel 18     
#define DMA_SADR18   		0x324		// DMA Source Address Register channel 18     
#define DMA_TADR18   		0x328		// DMA Target Address Register channel 18     
#define DMA_CMD18    		0x32c		// DMA Command Address Register channel 18     
#define DMA_DADR19   		0x330		// DMA Descriptor Address Register channel 19     
#define DMA_SADR19   		0x334		// DMA Source Address Register channel 19     
#define DMA_TADR19   		0x338		// DMA Target Address Register channel 19     
#define DMA_CMD19    		0x33c		// DMA Command Address Register channel 19     
#define DMA_DADR20   		0x340		// DMA Descriptor Address Register channel 20     
#define DMA_SADR20   		0x344		// DMA Source Address Register channel 20     
#define DMA_TADR20   		0x348		// DMA Target Address Register channel 20     
#define DMA_CMD20    		0x34c		// DMA Command Address Register channel 20     
#define DMA_DADR21   		0x350		// DMA Descriptor Address Register channel 21     
#define DMA_SADR21   		0x354		// DMA Source Address Register channel 21     
#define DMA_TADR21   		0x358		// DMA Target Address Register channel 21     
#define DMA_CMD21    		0x35c		// DMA Command Address Register channel 21     
#define DMA_DADR22   		0x360		// DMA Descriptor Address Register channel 22     
#define DMA_SADR22   		0x364		// DMA Source Address Register channel 22     
#define DMA_TADR22   		0x368		// DMA Target Address Register channel 22     
#define DMA_CMD22    		0x36c		// DMA Command Address Register channel 22     
#define DMA_DADR23   		0x370		// DMA Descriptor Address Register channel 23     
#define DMA_SADR23   		0x374		// DMA Source Address Register channel 23     
#define DMA_TADR23   		0x378		// DMA Target Address Register channel 23     
#define DMA_CMD23    		0x37c		// DMA Command Address Register channel 23     
#define DMA_DADR24   		0x380		// DMA Descriptor Address Register channel 24     
#define DMA_SADR24   		0x384		// DMA Source Address Register channel 24     
#define DMA_TADR24   		0x388		// DMA Target Address Register channel 24     
#define DMA_CMD24    		0x38c		// DMA Command Address Register channel 24     
#define DMA_DADR25   		0x390		// DMA Descriptor Address Register channel 25     
#define DMA_SADR25   		0x394		// DMA Source Address Register channel 25     
#define DMA_TADR25   		0x398		// DMA Target Address Register channel 25     
#define DMA_CMD25    		0x39c		// DMA Command Address Register channel 25     
#define DMA_DADR26   		0x3a0		// DMA Descriptor Address Register channel 26     
#define DMA_SADR26   		0x3a4		// DMA Source Address Register channel 26     
#define DMA_TADR26   		0x3a8		// DMA Target Address Register channel 26     
#define DMA_CMD26    		0x3ac		// DMA Command Address Register channel 26     
#define DMA_DADR27   		0x3b0		// DMA Descriptor Address Register channel 27     
#define DMA_SADR27   		0x3b4		// DMA Source Address Register channel 27     
#define DMA_TADR27   		0x3b8		// DMA Target Address Register channel 27     
#define DMA_CMD27    		0x3bc		// DMA Command Address Register channel 27     
#define DMA_DADR28   		0x3c0		// DMA Descriptor Address Register channel 28     
#define DMA_SADR28   		0x3c4		// DMA Source Address Register channel 28     
#define DMA_TADR28   		0x3c8		// DMA Target Address Register channel 28     
#define DMA_CMD28    		0x3cc		// DMA Command Address Register channel 28     
#define DMA_DADR29   		0x3d0		// DMA Descriptor Address Register channel 29     
#define DMA_SADR29   		0x3d4		// DMA Source Address Register channel 29     
#define DMA_TADR29   		0x3d8		// DMA Target Address Register channel 29     
#define DMA_CMD29    		0x3dc		// DMA Command Address Register channel 29     
#define DMA_DADR30   		0x3e0		// DMA Descriptor Address Register channel 30     
#define DMA_SADR30   		0x3e4		// DMA Source Address Register channel 30     
#define DMA_TADR30   		0x3e8		// DMA Target Address Register channel 30     
#define DMA_CMD30    		0x3ec		// DMA Command Address Register channel 30     
#define DMA_DADR31   		0x3f0		// DMA Descriptor Address Register channel 31     
#define DMA_SADR31   		0x3f4		// DMA Source Address Register channel 31     
#define DMA_TADR31   		0x3f8		// DMA Target Address Register channel 31     
#define DMA_CMD31    		0x3fc		// DMA Command Address Register channel 31     
#define DMA_DRCMR64  		0x1100		// Request to Channel Map Register for Memory Stick receive Request
#define DMA_DRCMR65  		0x1104		// Request to Channel Map Register for Memory Stick transmit Request 
#define DMA_DRCMR66  		0x1105		// Request to Channel Map Register for SSP3 receive Request
#define DMA_DRCMR67  		0x110c		// Request to Channel Map Register for SSP3 transmit Request  
#define DMA_DRCMR68  		0x1110		// Request to Channel Map Register for Quick Capture Interface Receive Request 0
#define DMA_DRCMR69  		0x1114		// Request to Channel Map Register for Quick Capture Interface Receive Request 1
#define DMA_DRCMR70  		0x1118		// Request to Channel Map Register for Quick Capture Interface Receive Request 2
#define DMA_DRCMR71  		0x111c		// Reserved
#define DMA_DRCMR72  		0x1120		// Reserved
#define DMA_DRCMR73  		0x1124		// Reserved
#define DMA_DRCMR74  		0x1128		// Request to Channel Map Register for DREQ 2 (companion chip request 2)


//---------------------------------------
//     UARTs
#define FF_UART_BASE		0x40100000	// Full Function UART Base Address
#define FF_UART_REG(_x_)	*(vulong *)(FF_UART_BASE + _x_)	
#define BT_UART_BASE		0x40200000	// Bluetooth UART Base Address
#define BT_UART_REG(_x_)	*(vulong *)(BT_UART_BASE + _x_)	
#define STD_UART_BASE		0x40700000	// Standard UART Base Address
#define STD_UART_REG(_x_)	*(vulong *)(STD_UART_BASE + _x_)	

#define UART_RBR      		0x00        // Receive Buffer Register (read only)     
#define UART_THR      		0x00        // Transmit Holding Register (write only)     
#define UART_DLL      		0x00        // Baud divisor lower byte (read/write)     
#define UART_IER      		0x04        // Interrupt Enable Register (read/write)     
#define UART_DLH      		0x04        // Baud divisor higher byte (read/write)     
#define UART_IIR      		0x08        // Interrupt ID Register (read only)     
#define UART_FCR      		0x08        // FIFO Control Register (write only)     
#define UART_LCR      		0x0C        // Line Control Register (read/write)     
#define UART_MCR      		0x10        // Modem Control Register (read/write)     
#define UART_LSR      		0x14        // Line Status Register (read only)     
#define UART_MSR      		0x18        // Modem Status Register (read only)     
#define UART_SPR      		0x1C        // Scratch Pad Register (read/write)     
#define UART_ISR      		0x20        // Infrared Select Register (read/write)     
#define UART_FOR      		0x24        // Receive FIFO Occupancy Register (read/write)     
#define UART_ABR      		0x25        // Auto-baud Control Register (read/write)     
#define UART_ACR      		0x2C        // Auto-baud Count Register (read/write)     

//---------------------------------------
//     I2C
#define I2C_BASE			0x40301680
#define I2C_REG(_x_)		*(vulong *)(I2C_BASE + _x_)	

#define I2C_IBMR			0x00		// I2C Bus Monitor Register - IBMR.
#define I2C_IDBR			0x08		// I2C Data Buffer Register - IDBR.
#define I2C_ICR				0x10		// I2C Control Register - ICR.
#define I2C_ISR				0x18		// I2C Status Register - ISR.
#define I2C_ISAR			0x20		// I2C Slave Address Register - ISAR.

#define CKEN14_I2C			(1 << 14)	// I2C Unit Clock Enable.

//---------------------------------------
//     I2S
#define I2S_BASE			0x40400000   
#define I2S_REG(_x_)		*(vulong *)(I2S_BASE + _x_)	
#define I2S_SACR0      		0x00        // Global Control Register
#define I2S_SACR1      		0x04        // Serial Audio I2S/MSB-Justified Control Register
#define I2S_SASR0      		0x0C        // Serial Audio I2S/MSB-Justified Interface and FIFO Status Register
#define I2S_SAIMR      		0x14        // Serial Audio Interrupt Mask Register
#define I2S_SAICR      		0x18        // Serial Audio Interrupt Clear Register
#define I2S_SADIV      		0x60        // Audio clock divider register. See section Section 12.3
										// Serial Audio Clocks and Sampling Frequencies on page 12-7.
#define I2S_SADR       		0x80        // Serial Audio Data Register (TX and RX FIFO access register).

//---------------------------------------
//     AC97
#define AC97_BASE			0x40500000   
#define AC97_REG(_x_)		*(vulong *)(AC97_BASE + _x_)	
#define AC97_POCR        	0x000       // PCM Out Control Register
#define AC97_PICR        	0x004       // PCM In Control Register
#define AC97_MCCR        	0x008       // Mic In Control Register
#define AC97_GCR         	0x00C       // Global Control Register 
#define AC97_POSR        	0x010       // PCM Out Status Register
#define AC97_PISR        	0x014       // PCM In Status Register
#define AC97_MCSR        	0x018       // Mic In Status Register
#define AC97_GSR         	0x01C       // Global Status Register 
#define AC97_CAR         	0x020       // CODEC Access Register 
#define AC97_PCDR       	0x040       // PCM FIFO Data Register 
#define AC97_MCDR       	0x060       // Mic-in FIFO Data Register 
#define AC97_MOCR       	0x100       // MODEM Out Control Register
#define AC97_MICR       	0x108       // MODEM In Control Register
#define AC97_MOSR       	0x110       // MODEM Out Status Register
#define AC97_MISR       	0x118       // MODEM In Status Register
#define AC97_MODR       	0x140       // MODEM FIFO Data Register

//---------------------------------------
//     USB Client
#define USB_BASE			0x40600000   
#define USB_REG(_x_)		*(vulong *)(USB_BASE + _x_)	
#define USB_UDCCR      		0x000          	// UDC control register
#define USB_UICR0    		0x004          	// UDC Interrupt Control Register 0
#define USB_UICR1    		0x008          	// UDC Interrupt Control Register 1
#define USB_USIR0    		0x00C          	// UDC Interrupt Status Register 0
#define USB_USIR1    		0x010          	// UDC Interrupt Status Register 1
#define USB_UFNR     		0x014          	// UDC Frame Number Register
#define USB_UOTGICR    		0x018          	// UDC OTG Interrupt Control Register
#define USB_UOTGISR    		0x01C          	// UDC OTG Interrupt Status Register
#define USB_UP2OCR     		0x020          	// USB Port 2 Output Control Register
#define USB_UP3OCR     		0x024          	// USB Port 3 Output Control Register
#define USB_UDCCS0     		0x100          	// UDC Endpoint 0 Control/Status Register
#define USB_UDCCS1     		0x104          	// UDC Endpoint A Control/Status Register
#define USB_UDCCS2     		0x108          	// UDC Endpoint B Control/Status Register
#define USB_UDCCS3     		0x10C          	// UDC Endpoint C Control/Status Register
#define USB_UDCCS4     		0x110          	// UDC Endpoint D Control/Status Register
#define USB_UDCCS5     		0x114          	// UDC Endpoint E Control/Status Register
#define USB_UDCCS6     		0x118          	// UDC Endpoint F Control/Status Register
#define USB_UDCCS7     		0x11C          	// UDC Endpoint G Control/Status Register
#define USB_UDCCS8    		0x120          	// UDC Endpoint H Control/Status Register
#define USB_UDCCS9    		0x124          	// UDC Endpoint I Control/Status Register
#define USB_UDCCS10    		0x128          	// UDC Endpoint J Control/Status Register
#define USB_UDCCS11    		0x12C          	// UDC Endpoint K Control/Status Register
#define USB_UDCCS12    		0x130          	// UDC Endpoint L Control/Status Register
#define USB_UDCCS13    		0x134          	// UDC Endpoint M Control/Status Register
#define USB_UDCCS14    		0x138          	// UDC Endpoint N Control/Status Register
#define USB_UDCCS15    		0x13C          	// UDC Endpoint P Control/Status Register
#define USB_UDCCS16    		0x140          	// UDC Endpoint Q Control/Status Register
#define USB_UDCCS17    		0x144          	// UDC Endpoint R Control/Status Register
#define USB_UDCCS18    		0x148          	// UDC Endpoint S Control/Status Register
#define USB_UDCCS19    		0x14C          	// UDC Endpoint T Control/Status Register
#define USB_UDCCS20    		0x150          	// UDC Endpoint U Control/Status Register
#define USB_UDCCS21    		0x154          	// UDC Endpoint V Control/Status Register
#define USB_UDCCS22    		0x158          	// UDC Endpoint W Control/Status Register
#define USB_UDCCS23    		0x15C          	// UDC Endpoint X Control/Status Register
#define USB_UDCBC0     		0x200          	// UDC Endpoint 0 Byte Count Register
#define USB_UDCBC1     		0x204          	// UDC Endpoint A Byte Count Register
#define USB_UDCBC2     		0x208          	// UDC Endpoint B Byte Count Register
#define USB_UDCBC3     		0x20C          	// UDC Endpoint C Byte Count Register
#define USB_UDCBC4     		0x210          	// UDC Endpoint D Byte Count Register
#define USB_UDCBC5     		0x214          	// UDC Endpoint E Byte Count Register
#define USB_UDCBC6     		0x218          	// UDC Endpoint F Byte Count Register
#define USB_UDCBC7     		0x21C          	// UDC Endpoint G Byte Count Register
#define USB_UDCBC8    		0x220          	// UDC Endpoint H Byte Count Register
#define USB_UDCBC9    		0x224          	// UDC Endpoint I Byte Count Register
#define USB_UDCBC10    		0x228          	// UDC Endpoint J Byte Count Register
#define USB_UDCBC11    		0x22C          	// UDC Endpoint K Byte Count Register
#define USB_UDCBC12    		0x230          	// UDC Endpoint L Byte Count Register
#define USB_UDCBC13    		0x234          	// UDC Endpoint M Byte Count Register
#define USB_UDCBC14    		0x238          	// UDC Endpoint N Byte Count Register
#define USB_UDCBC15    		0x23C          	// UDC Endpoint P Byte Count Register
#define USB_UDCBC16    		0x240          	// UDC Endpoint Q Byte Count Register
#define USB_UDCBC17    		0x244          	// UDC Endpoint R Byte Count Register
#define USB_UDCBC18    		0x248          	// UDC Endpoint S Byte Count Register
#define USB_UDCBC19    		0x24C          	// UDC Endpoint T Byte Count Register
#define USB_UDCBC20    		0x250          	// UDC Endpoint U Byte Count Register
#define USB_UDCBC21    		0x254          	// UDC Endpoint V Byte Count Register
#define USB_UDCBC22    		0x258          	// UDC Endpoint W Byte Count Register
#define USB_UDCBC23    		0x25C          	// UDC Endpoint X Byte Count Register
#define USB_UDDR0      		0x300          	// UDC Endpoint 0 Data Register
#define USB_UDDR1      		0x304          	// UDC Endpoint A Data Register
#define USB_UDDR2      		0x308          	// UDC Endpoint B Data Register
#define USB_UDDR3      		0x30C          	// UDC Endpoint C Data Register
#define USB_UDDR4      		0x310          	// UDC Endpoint D Data Register
#define USB_UDDR5      		0x314          	// UDC Endpoint E Data Register
#define USB_UDDR6      		0x318          	// UDC Endpoint F Data Register
#define USB_UDDR7      		0x31C          	// UDC Endpoint G Data Register
#define USB_UDDR8      		0x320          	// UDC Endpoint H Data Register
#define USB_UDDR9      		0x324          	// UDC Endpoint I Data Register
#define USB_UDDR10     		0x328          	// UDC Endpoint J Data Register
#define USB_UDDR11     		0x32C          	// UDC Endpoint K Data Register
#define USB_UDDR12     		0x330          	// UDC Endpoint L Data Register
#define USB_UDDR13     		0x334          	// UDC Endpoint M Data Register
#define USB_UDDR14     		0x338          	// UDC Endpoint N Data Register
#define USB_UDDR15     		0x33C          	// UDC Endpoint P Data Register
#define USB_UDDR16     		0x340          	// UDC Endpoint Q Data Register
#define USB_UDDR17     		0x344          	// UDC Endpoint R Data Register
#define USB_UDDR18     		0x348          	// UDC Endpoint S Data Register
#define USB_UDDR19     		0x34C          	// UDC Endpoint T Data Register
#define USB_UDDR20     		0x350          	// UDC Endpoint U Data Register
#define USB_UDDR21     		0x354          	// UDC Endpoint V Data Register
#define USB_UDDR22     		0x358          	// UDC Endpoint W Data Register
#define USB_UDDR23     		0x35C          	// UDC Endpoint X Data Register
#define USB_UDCR0      		0x400          	// Reserved
#define USB_UDCR1      		0x404          	// UDC Endpoint A Configuration Register
#define USB_UDCR2      		0x408          	// UDC Endpoint B Configuration Register
#define USB_UDCR3      		0x40C          	// UDC Endpoint C Configuration Register
#define USB_UDCR4      		0x410          	// UDC Endpoint D Configuration Register
#define USB_UDCR5      		0x414          	// UDC Endpoint E Configuration Register
#define USB_UDCR6      		0x418          	// UDC Endpoint F Configuration Register
#define USB_UDCR7      		0x41C          	// UDC Endpoint G Configuration Register
#define USB_UDCR8      		0x420          	// UDC Endpoint H Configuration Register
#define USB_UDCR9      		0x424          	// UDC Endpoint I Configuration Register
#define USB_UDCR10     		0x428          	// UDC Endpoint J Configuration Register
#define USB_UDCR11     		0x42C          	// UDC Endpoint K Configuration Register
#define USB_UDCR12     		0x430          	// UDC Endpoint L Configuration Register
#define USB_UDCR13     		0x434          	// UDC Endpoint M Configuration Register
#define USB_UDCR14     		0x438          	// UDC Endpoint N Configuration Register
#define USB_UDCR15     		0x43C          	// UDC Endpoint P Configuration Register
#define USB_UDCR16     		0x440          	// UDC Endpoint Q Configuration Register
#define USB_UDCR17     		0x444          	// UDC Endpoint R Configuration Register
#define USB_UDCR18     		0x448          	// UDC Endpoint S Configuration Register
#define USB_UDCR19     		0x44C          	// UDC Endpoint T Configuration Register
#define USB_UDCR20     		0x450          	// UDC Endpoint U Configuration Register
#define USB_UDCR21     		0x454          	// UDC Endpoint V Configuration Register
#define USB_UDCR22     		0x458          	// UDC Endpoint W Configuration Register
#define USB_UDCR23     		0x45C          	// UDC Endpoint X Configuration Register

//---------------------------------------
//	   Fast Infrared UART	
#define IRDA_BASE			0x40800000  
#define IRDA_REG(_x_)		*(vulong *)(IRDA_BASE + _x_)	
#define IRDA_ICCR0      	0x00          	// FICP control register 0
#define IRDA_ICCR1      	0x04          	// FICP control register 1
#define IRDA_ICCR2      	0x08          	// FICP control register 2
#define IRDA_ICDR       	0x0C          	// FICP data register
#define IRDA_ICSR0      	0x14          	// FICP status register 0
#define IRDA_ICSR1      	0x18          	// FICP status register 1
#define IRDA_ICFOR      	0x1C          	// FICP FIFO Occupancy Status register

//---------------------------------------
//     RTC
#define RTC_BASE			0x40900000   
#define RTC_REG(_x_)		*(vulong *)(RTC_BASE + _x_)	
#define RTC_RCNR       		0x00          	// RTC count register
#define RTC_RTAR       		0x04          	// RTC alarm register
#define RTC_RTSR       		0x08          	// RTC status register
#define RTC_RTTR       		0x0C          	// RTC timer trim register
#define RTC_RDCR       		0x10          	// RTC day counter register
#define RTC_RYCR       		0x14          	// RTC year counter register
#define RTC_RDAR1      		0x18          	// RTC wristwatch day alarm register 1
#define RTC_RYAR1      		0x1C          	// RTC wristwatch year alarm register 1
#define RTC_RDAR2      		0x20          	// RTC wristwatch day alarm register 2
#define RTC_RYAR2      		0x24          	// RTC wristwatch year alarm register 2
#define RTC_RWCR       		0x28          	// RTC stopwatch counter register
#define RTC_RWAR1      		0x2C          	// RTC stopwatch alarm register 1
#define RTC_RWAR2      		0x30          	// RTC stopwatch alarm register	2
#define RTC_RPICR      		0x34          	// RTC periodic interrupt counter register
#define RTC_RPIAR      		0x38          	// RTC periodic interrupt alarm register

//---------------------------------------
//     OS Timer
#define TMR_BASE			0x40A00000   
#define TMR_REG(_x_)		*(vulong *)(TMR_BASE + _x_)	
#define TMR_OSMR0      		0x00          	// OS timer match registers<3:0>
#define TMR_OSMR1      		0x04          	//
#define TMR_OSMR2      		0x08          	//
#define TMR_OSMR3      		0x0C          	//
#define TMR_OSCR0      		0x10          	// OS timer counter 0 register
#define TMR_OSSR       		0x14          	// OS timer status register
#define TMR_OWER       		0x18          	// OS timer watchdog enable register
#define TMR_OIER       		0x1C          	// OS timer interrupt enable register
#define TMR_OSNR       		0x20          	// OS timer snapshot register
#define TMR_OSCR4      		0x40          	// OS timer counter 4-11 registers
#define TMR_OSCR5      		0x44          	// 
#define TMR_OSCR6      		0x48          	// 
#define TMR_OSCR7      		0x4C          	// 
#define TMR_OSCR8      		0x50          	// 
#define TMR_OSCR9      		0x54          	// 
#define TMR_OSCR10     		0x58          	// 
#define TMR_OSCR11     		0x5C          	// 
#define TMR_OSMR4      		0x80          	// OS timer match 4-11 registers
#define TMR_OSMR5      		0x84          	// 
#define TMR_OSMR6      		0x88          	// 
#define TMR_OSMR7      		0x8C          	// 
#define TMR_OSMR8      		0x90          	// 
#define TMR_OSMR9      		0x94          	// 
#define TMR_OSMR10     		0x98          	// 
#define TMR_OSMR11     		0x9C          	// 
#define TMR_OMCR4      		0xC0          	// OS match control 4-11 registers
#define TMR_OMCR5      		0xC4          	// 
#define TMR_OMCR6      		0xC8          	// 
#define TMR_OMCR7      		0xCC          	// 
#define TMR_OMCR8      		0xD0          	// 
#define TMR_OMCR9      		0xD4          	// 
#define TMR_OMCR10     		0xD8          	// 
#define TMR_OMCR11     		0xDC          	// 



//---------------------------------------
//     PWM 
#define PWM0_BASE			0x40B00000  
#define PWM0_REG(_x_)		*(vulong *)(PWM0_BASE + _x_)	
#define PWM1_BASE			0x40C00000  
#define PWM1_REG(_x_)		*(vulong *)(PWM1_BASE + _x_)	
#define PWM2_BASE			0x40B00010  
#define PWM2_REG(_x_)		*(vulong *)(PWM0_BASE + _x_)	
#define PWM3_BASE			0x40C00010  
#define PWM3_REG(_x_)		*(vulong *)(PWM1_BASE + _x_)	
#define PWM_PWMCTRL   		0x00          	// PWM Control Register
#define PWM_PWDUTY    		0x04          	// PWM Duty Cycle Register
#define PWM_PERVAL    		0x08          	// PWM Period Control Register

//---------------------------------------
//     Interrupt Control   
#define INT_BASE   			0x40D00000      // Interrupt controller IRQ pending register
#define INT_REG(_x_)		*(vulong *)(INT_BASE + _x_)	
#define INT_ICIP       		0x00      		// Interrupt controller IRQ pending register
#define INT_ICMR       		0x04      		// Interrupt controller mask register
#define INT_ICLR       		0x08      		// Interrupt controller level register
#define INT_ICFP       		0x0C      		// Interrupt controller FIQ pending register
#define INT_ICPR       		0x10      		// Interrupt controller pending register
#define INT_ICCR       		0x14      		// Interrupt controller control register
#define INT_ICHP       		0x18      		// Interrupt controller highest priority register
#define INT_IPR0       		0x1C      		// Interrupt priority registers for Priorities 0-31
#define INT_IPR1       		0x20      		// 
#define INT_IPR2       		0x24      		// 
#define INT_IPR3       		0x28      		// 
#define INT_IPR4       		0x2C      		// 
#define INT_IPR5       		0x30      		// 
#define INT_IPR6       		0x34      		// 
#define INT_IPR7       		0x38      		// 
#define INT_IPR8       		0x3C      		// 
#define INT_IPR9       		0x40      		// 
#define INT_IPR10      		0x44      		// 
#define INT_IPR11      		0x48      		// 
#define INT_IPR12      		0x4C      		// 
#define INT_IPR13      		0x50      		// 
#define INT_IPR14      		0x54      		// 
#define INT_IPR15      		0x58      		// 
#define INT_IPR16      		0x5C      		// 
#define INT_IPR17      		0x60      		// 
#define INT_IPR18      		0x64      		// 
#define INT_IPR19      		0x68      		// 
#define INT_IPR20      		0x6C      		// 
#define INT_IPR21      		0x70      		// 
#define INT_IPR22      		0x74      		// 
#define INT_IPR23      		0x78      		// 
#define INT_IPR24      		0x7C      		// 
#define INT_IPR25      		0x80      		// 
#define INT_IPR26      		0x84      		// 
#define INT_IPR27      		0x88      		// 
#define INT_IPR28      		0x8C      		// 
#define INT_IPR29      		0x90      		// 
#define INT_IPR30      		0x94      		// 
#define INT_IPR31      		0x98      		// 
#define INT_ICIP2      		0x9C      		// Interrupt controller IRQ pending register 2
#define INT_ICMR2      		0xA0      		// Interrupt controller mask register 2
#define INT_ICLR2      		0xA4      		// Interrupt controller level register 2
#define INT_ICFP2      		0xA8      		// Interrupt controller FIQ pending register 
#define INT_ICPR2      		0xAC      		// Interrupt controller pending register 2
#define INT_IPR32      		0xB0      		// Interrupt priority registers for Priorities 32-39
#define INT_IPR33      		0xB4      		// 
#define INT_IPR34      		0xB8      		// 
#define INT_IPR35      		0xBC      		// 
#define INT_IPR36      		0xC0      		// 
#define INT_IPR37      		0xC4      		// 
#define INT_IPR38      		0xC8      		// 
#define INT_IPR39      		0xCC      		// 

//---------------------------------------
// GPIO - Register offsets and bit 
// defines are in pxa250_gpio.h
#define GPIO_BASE			0x40E00000
#define GPIO_REG(_x_)		*(vulong *)(GPIO_BASE + _x_)	

//---------------------------------------
//     Power Manager and Reset Control
#define PWR_CTL_BASE		0x40F00000  	 
#define PWR_REG(_x_)		*(vulong *)(PWR_CTL_BASE + _x_)	
#define PWR_PMCR       		0x00      // Power Manager Control register
#define PWR_PSSR       		0x04      // Power Manager Sleep Status register
#define PWR_PSPR       		0x08      // Power Manager Scratch Pad register
#define PWR_PWER       		0x0C      // Power Manager Wake-up Enable register
#define PWR_PRER       		0x10      // Power Manager GPIO Rising-edge Detect Enable register
#define PWR_PFER       		0x14      // Power Manager GPIO Falling-edge Detect Enable register
#define PWR_PEDR       		0x18      // Power Manager GPIO Edge Detect Status register
#define PWR_PCFR       		0x1C      // Power Manager General Configuration register
#define PWR_PGSR0      		0x20      // Power Manager GPIO Sleep State register for GP[31-0]
#define PWR_PGSR1      		0x24      // Power Manager GPIO Sleep State register for GP[63-32]
#define PWR_PGSR2      		0x28      // Power Manager GPIO Sleep State register for GP[95-64]
#define PWR_PGSR3      		0x2C      // Power Manager GPIO Sleep State register for GP[120-96]
#define PWR_RCSR       		0x30      // Reset controller status register
#define PWR_PSLR       		0x34      // Power Manager Sleep Configuration register
#define PWR_PSTR       		0x38      // Power Manager Standby Configuration register
#define PWR_PVCR       		0x40      // Power Manager Voltage Change Control register
#define PWR_PUCR       		0x4C      // Power Manager USIM Card Control/Status register
#define PWR_PKWR       		0x50      // Power Manager Keyboard Wake-up Enable register
#define PWR_PKSR       		0x48      // Power Manager Keyboard Level-Detect Status register
#define PWR_PCMD0      		0x80      // Power Manager I2C Command register file
#define PWR_PCMD1      		0x84      // Power Manager I2C Command register file
#define PWR_PCMD2      		0x88      // Power Manager I2C Command register file
#define PWR_PCMD3      		0x8C      // Power Manager I2C Command register file
#define PWR_PCMD4      		0x90      // Power Manager I2C Command register file
#define PWR_PCMD5      		0x94      // Power Manager I2C Command register file
#define PWR_PCMD6      		0x98      // Power Manager I2C Command register file
#define PWR_PCMD7      		0x9C      // Power Manager I2C Command register file
#define PWR_PCMD8      		0xA0      // Power Manager I2C Command register file
#define PWR_PCMD9      		0xA4      // Power Manager I2C Command register file
#define PWR_PCMD10     		0xA8      // Power Manager I2C Command register file
#define PWR_PCMD11     		0xAC      // Power Manager I2C Command register file
#define PWR_PCMD12     		0xB0      // Power Manager I2C Command register file
#define PWR_PCMD13     		0xB4      // Power Manager I2C Command register file
#define PWR_PCMD14     		0xB8      // Power Manager I2C Command register file
#define PWR_PCMD15     		0xBC      // Power Manager I2C Command register file
#define PWR_PCMD16     		0xC0      // Power Manager I2C Command register file
#define PWR_PCMD17     		0xC4      // Power Manager I2C Command register file
#define PWR_PCMD18     		0xC8      // Power Manager I2C Command register file
#define PWR_PCMD19     		0xCC      // Power Manager I2C Command register file
#define PWR_PCMD20     		0xD0      // Power Manager I2C Command register file
#define PWR_PCMD21     		0xD4      // Power Manager I2C Command register file
#define PWR_PCMD22     		0xD8      // Power Manager I2C Command register file
#define PWR_PCMD23     		0xDC      // Power Manager I2C Command register file
#define PWR_PCMD24     		0xE0      // Power Manager I2C Command register file
#define PWR_PCMD25     		0xE4      // Power Manager I2C Command register file
#define PWR_PCMD26     		0xE8      // Power Manager I2C Command register file
#define PWR_PCMD27     		0xEC      // Power Manager I2C Command register file
#define PWR_PCMD28     		0xF0      // Power Manager I2C Command register file
#define PWR_PCMD29     		0xF4      // Power Manager I2C Command register file
#define PWR_PCMD30     		0xF8      // Power Manager I2C Command register file
#define PWR_PCMD31     		0xFC      // Power Manager I2C Command register file

//---------------------------------------
//     Power Manager I2C
#define PWR_I2C_BASE		0x40F00180  	 
#define PWR_I2C_REG(_x_)	*(vulong *)(PWR_I2C_BASE + _x_)	
#define PWR_IBMR       		0x00      // Power Manager I2C Bus Monitor register
#define PWR_IDBR       		0x08      // Power Manager I2C Data Buffer register
#define PWR_ICR       		0x10      // Power Manager I2C Control register
#define PWR_ISR       		0x18      // Power Manager I2C Status register
#define PWR_ISAR       		0x20      // Power Manager I2C Slave Address register

//---------------------------------------
//     Clocks Manager
#define CLK_BASE			0x41300000   
#define CLK_REG(_x_)		*(vulong *)(CLK_BASE + _x_)	
#define CLK_CCCR       		0x00          // Core Clock Configuration Register
#define CLK_CKEN       		0x04          // Clock Enable Register
#define CLK_OSCC       		0x08          // Oscillator Configuration Register
#define CLK_CCSR       		0x0C          // Core Clock Status Register

//---------------------------------------
//     SSP Port 1
#define SSP_BASE			0x41000000  
#define SSP_REG(_x_)		*(vulong *)(SSP_BASE + _x_)	
#define SSP_SSCR0      		0x00          	// SSP 1 Control Register 0
#define SSP_SSCR1      		0x04          	// SSP 1 Control Register 1 
#define SSP_SSSR       		0x08          	// SSP 1 Status Register
#define SSP_SSITR      		0x0C          	// SSP 1 Interrupt Test Register
#define SSP_SSDR       		0x10          	// SSP 1 Data Write Register/SSP Data Read Register
#define SSP_SSTO       		0x28          	// SSP 1 Time-Out Register
#define SSP_SSPSP      		0x2C          	// SSP 1 Programmable Serial Protocol
#define SSP_SSTSA      		0x30          	// SSP 1 TX Timeslot Active Register
#define SSP_SSRSA      		0x34          	// SSP 1 RX Timeslot Active Register 
#define SSP_SSTSS      		0x38          	// SSP 1 Timeslot Status Register
#define SSP_SSACD      		0x3C          	// SSP 1 Audio Clock Divider Register

//---------------------------------------
//     MMC 
#define MMC_BASE			0x41100000  
#define MMC_REG(_x_)		*(vulong *)(MMC_BASE + _x_)	
#define MMC_MMCSTRPCL  		0x00          	// Control to start and stop MMC clock	
#define MMC_MMCSTAT    		0x04          	// MMC status register (read only)	
#define MMC_MMCCLKRT   		0x08          	// MMC clock rate 	
#define MMC_MMCSPI     		0x0c          	// SPI mode control bits	
#define MMC_MMCCMDAT   		0x10          	// Command/response/data sequence control	
#define MMC_MMCRESTO   		0x14          	// Expected response time out	
#define MMC_MMCRDTO    		0x18          	// Expected data read time out	
#define MMC_MMCBLKLEN  		0x1c          	// Block length of data transaction	
#define MMC_MMCNOB     		0x20          	// "Number of blocksfor block mode"	
#define MMC_MMCPRTBUF  		0x24          	// Partial MMC_TXFIFO FIFO written	
#define MMC_MMCIMASK   		0x28          	// Interrupt Mask	
#define MMC_MMCIREG    		0x2c          	// Interrupt Register (read only)	
#define MMC_MMCCMD     		0x30          	// Index of current command	
#define MMC_MMCARGH    		0x34          	// MSW part of the current command argument	
#define MMC_MMCARGL    		0x38          	// LSW part of the current command argument	
#define MMC_MMCRES     		0x3c          	// Response FIFO (read only)	
#define MMC_MMCRXFIFO  		0x40          	// Receive FIFO (read only)	
#define MMC_MMCTXFIFO  		0x44          	// Transmit FIFO (write only)	
#define MMC_MMCRDWAIT  		0x48          	// RD_WAIT Register	
#define MMC_MMCBLKSREM 		0x4C          	// Blocks remaining register	

//---------------------------------------
//     SSP Port 2
#define SSP2_BASE			0x41700000  
#define SSP2_REG(_x_)		*(vulong *)(SSP_BASE + _x_)	
#define SSP2_SSCR0      		0x00          	// SSP 2 Control Register 0
#define SSP2_SSCR1      		0x04          	// SSP 2 Control Register 1 
#define SSP2_SSSR       		0x08          	// SSP 2 Status Register
#define SSP2_SSITR      		0x0C          	// SSP 2 Interrupt Test Register
#define SSP2_SSDR       		0x10          	// SSP 2 Data Write Register/SSP Data Read Register
#define SSP2_SSTO       		0x28          	// SSP 2 Time-Out Register
#define SSP2_SSPSP      		0x2C          	// SSP 2 Programmable Serial Protocol
#define SSP2_SSTSA      		0x30          	// SSP 2 TX Timeslot Active Register
#define SSP2_SSRSA      		0x34          	// SSP 2 RX Timeslot Active Register 
#define SSP2_SSTSS      		0x38          	// SSP 2 Timeslot Status Register
#define SSP2_SSACD      		0x3C          	// SSP 2 Audio Clock Divider Register

//---------------------------------------
//     SSP Port 3
#define SSP3_BASE			0x41900000  
#define SSP3_REG(_x_)		*(vulong *)(SSP_BASE + _x_)	
#define SSP3_SSCR0      		0x00          	// SSP 3 Control Register 0
#define SSP3_SSCR1      		0x04          	// SSP 3 Control Register 1 
#define SSP3_SSSR       		0x08          	// SSP 3 Status Register
#define SSP3_SSITR      		0x0C          	// SSP 3 Interrupt Test Register
#define SSP3_SSDR       		0x10          	// SSP 3 Data Write Register/SSP Data Read Register
#define SSP3_SSTO       		0x28          	// SSP 3 Time-Out Register
#define SSP3_SSPSP      		0x2C          	// SSP 3 Programmable Serial Protocol
#define SSP3_SSTSA      		0x30          	// SSP 3 TX Timeslot Active Register
#define SSP3_SSRSA      		0x34          	// SSP 3 RX Timeslot Active Register 
#define SSP3_SSTSS      		0x38          	// SSP 3 Timeslot Status Register
#define SSP3_SSACD      		0x3C          	// SSP 3 Audio Clock Divider Register

//------------------------------------------------
// Bring in the individual peripheral files here
#include "pxa270_msc.h"
#include "pxa270_gpio.h"

