#include "config.h"
#include "cpuio.h"
#include "stddefs.h"
#include "mpc860.h"
#include "ether.h"
#include "genlib.h"
#include "arch_ppc.h"

EPPC	*IMMRBase;

/* Two leds, controlled by 2 bits...
 */
void
led_on(void)
{
	*(unsigned long *)0x30000000 = 0x0;
}

void
led_off(void)
{
	*(unsigned long *)0x30000000 = 0x00030003;
}

void
led_blink(void)
{
	while(1) {
		led_on();
		monDelay(300);
		led_off();
		monDelay(300);
	}
}

/* InitUART():
*/
void
InitUART(int baud)
{
	volatile int	timeout;
	volatile struct smc_uart_bd	*rbd, *tbd;
	volatile unsigned char *bcsr0;

	/* Set IMMRBase pointing to the MPC860's internal register memory map.
	 */
   	IMMRBase = (EPPC *)(ppcMfimmr() & 0xFFFF0000);

	/* Set up the pin (muxed with PIP) to be SMTXD1/SMRXD1:
	 * (see page 16-460 of MPC860 Users Manual)
	 */
	IMMRBase->pip_pbpar |= 0x000000c0;     
	IMMRBase->pip_pbdir &= 0xffffff3f;
	IMMRBase->pip_pbodr &= 0xff3f;

	/* Set up SI Mode Register to assign BRG1 to SMC1 (NMSI mode).
	 * (see page 16-114 of MPC860 Users Manual)
	 */
	IMMRBase->si_simode &= ~0x0000f000;	

	/* SDMA Control Register:
	 */
	IMMRBase->dma_sdcr = 0x0001;

	/* SMC Mode register  (pg 16-352 MPC860 Users Manual):
	 * 8 data bits, 1 stop bit, no parity
	 * TE & RE not yet enabled.
	 */
	IMMRBase->smc_regs[SMC1_REG].smc_smcmr = 0x4820;

	/* Set up a very basic buffer descriptor configuration.
	 * Both transmit and receive will use only a 1-byte buffer, both of
	 * these buffers will be in DPRAM.
	 */
	rbd = &IMMRBase->qcp_or_ud.umd.u_rbd;
	tbd = &IMMRBase->qcp_or_ud.umd.u_tbd;

	/* Set parameter ram to point to buffer descriptors in DPRAM:
	 * Note that this will generate a compiler warning because we are
	 * casting a pointer to a short. This can be ignored because these
	 * two entries are offsets relative to the base of the IMMR, not the
	 * actual address.
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.rbase = (ushort)rbd;
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.tbase = (ushort)tbd;

	/* Set RFCR,TFCR -- Rx,Tx Function Code
	 * Normal Operation and Motorola byte ordering
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.rfcr = 0x10;  
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.tfcr = 0x10;  

	/* MRBLR = MAX buffer length
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.mrblr = 1;    

	/* MAX_IDL = Disable MAX Idle Feature
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.max_idl = 0;  
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.brkln = 0;    

	/* BRKEC = No break condition occurred
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.brkec = 0;    

	/* BRKCR = 1 break char sent on top XMIT
	 */
	IMMRBase->PRAM[PAGE3].pg.other.smc_dsp1.psmc1.u1.brkcr = 1;    

	/* Mask all interrupts:
	 */
	IMMRBase->smc_regs[SMC1_REG].smc_smcm = 0;     
	IMMRBase->cpmi_cicr = 0;           /* Disable all CPM interrups */
	IMMRBase->cpmi_cipr = 0xFFFFFFFF;  /* Clear all pending interrupt events */
	IMMRBase->cpmi_cimr = 0;           /* Mask all event interrupts */
	
	/* Initialize buffer descriptors:
	 */
	rbd->status = 0xa000;		/* 16-354 */
	rbd->length = 1;
	rbd->baddr = IMMRBase->qcp_or_ud.umd.u_rxbuf;

	tbd->status = 0x2000;		/* 16-359 */
	tbd->length = 1;
	tbd->baddr = IMMRBase->qcp_or_ud.umd.u_txbuf;

	/* Clear any events:
	 */
	IMMRBase->smc_regs[SMC1_REG].smc_smce = 0xFF;

	/* Issue CPM Command to init RX/TX params (wait for completion):
	 */
	IMMRBase->cp_cr = 0x0091;
	timeout = 500000;
	while(IMMRBase->cp_cr & 0x0001) {
		timeout--;
		if (timeout <= 0)
			break;
	}

	IMMRBase->brgc1 = (0x10000 | ((CPU_CLOCK/(baud*32)) << 1));

//IMMRBase->brgc1 = 0x10034;	/* 65588 */

	/* note: this equates to a baud_clock value of 24883200 ??? */

	/* SMC Mode register  (pg 16-352 MPC860 Users Manual): */
	/* TE & RE now enabled. */
	IMMRBase->smc_regs[SMC1_REG].smc_smcmr = 0x4823;

	bcsr0 = (uchar *)((IMMRBase->memc_br3 & 0xffff8000));
	*bcsr0 |= 0x01;
}

int
ConsoleBaudSet(int baud)
{
	volatile int baud_clock;

	/* Setup baudrate based on incoming baud value:
	 * Clock is 4Mhz multplied up based on upper 12 bits of PLPRCR.
	 * (0x10000 | (((baud_clock/(baud*16)) - 1) << 1))
	 */
	baud_clock =
		(((IMMRBase->clkr_plprcr & 0xfff00000) >> 20) + 1) * CPU_CLOCK;
	IMMRBase->brgc1 = (0x10000 | (((baud_clock/(baud*16)) - 1) << 1));
	return(0);
}

int
devInit(int baud)
{
	InitUART(baud);
	return(0);
}

int
target_putchar(char c)
{
	volatile struct smc_uart_bd *tbd;

	tbd = &IMMRBase->qcp_or_ud.umd.u_tbd;

	/* Wait for ready bit to clear... */
	while(1) {
		if (!(tbd->status & 0x8000))
			break;
	}
	*tbd->baddr = (uchar)c;
	tbd->status |= 0x8000;
	return((int)c);
}

int
target_console_empty(void)
{
	volatile struct smc_uart_bd *tbd;

	tbd = &IMMRBase->qcp_or_ud.umd.u_tbd;
	if (!(tbd->status & 0x8000))
		return(1);
	return(0);
}

int	
target_getchar(void)
{
	char	c;
	volatile struct	smc_uart_bd *rbd;

	/* Retrieve character and clear status bit: */
	rbd = &IMMRBase->qcp_or_ud.umd.u_rbd;
	c = *rbd->baddr;
	rbd->status |= 0x8000;
	return((int)c);
}

int
target_gotachar(void)
{
	static volatile int ledtoggle, ledon;

	if (++ledtoggle > 50000) {
		if (ledon == 1) {
			led_off();
			ledon = 0;
		}
		else {
			led_on();
			ledon = 1;
		}
		ledtoggle = 0;
	}

	if (IMMRBase->qcp_or_ud.umd.u_rbd.status & 0x8000)
		return(0);
	else 
		return(1);
}

char *
portSize(unsigned long ps)
{
	switch(ps) {
		case 0x00000c00:
			return("rsvd");
		case 0x00000000:
			return("32");
		case 0x00000400:
			return(" 8");
		case 0x00000800:
			return("16");
	}
	return("???");
}

char *
CSDeviceNames[] = {
	"Boot Flash",
	"DRAM",
	"",
	"CPLD",
	"FPGA",
	"EEPROM",
	"",
	"",
};

void
CSInfo()
{
	int		i;
	vulong	*br, *or;

	/* Set IMMRBase pointing to the MPC866's internal register memory map. */
   	IMMRBase = (EPPC *)(ppcMfimmr() & 0xFFFF0000);

	printf("\n%s Chip Select Ctrl-Regs:\n",CPU_NAME);
	br = &IMMRBase->memc_br0;
	or = &IMMRBase->memc_or0;
	for(i=0;i<8;i++,br+=2,or+=2) {
		printf(" CS%d %s...\n",i,CSDeviceNames[i]);
		printf("   BR%d: 0x%08lx, OR%d: 0x%08lx\n",i,*br,i,*or);
		printf("   Base: 0x%08lx %s, %s bit, read-%s\n",
		    (ulong)(*br & 0xffff8000),
			(*br & 0x00000001) ? "  valid" : "invalid",
			portSize(*br & 0x00000c00),
		    (*br & 0x00000100) ? "only " : "write");
	}
}

void
initCPUio()
{
	uchar	*cp, *end;

	/* Set IMMRBase pointing to the MPC860's internal register memory map.
	 * The IMMR register should have already been set in reset.s; so here
	 * we are just grabbing a copy for C-use.
	 */
   	IMMRBase = (EPPC *)(ppcMfimmr() & 0xFFFF0000);
	
	/* Clear the internal RAM of the MPC860: */
	cp = (uchar *)&(IMMRBase->qcp_or_ud.umd);
	end = cp + 0x2000;
	while(cp < end)
		*cp++ = 0;
}

void
target_reset(void)
{
	coldstart();
}

void
intsrestore(ulong msr)
{
	ppcMtmsr(msr);
}

ulong
intsoff(void)
{
	ulong  msr;

	msr = ppcMfmsr();
	ppcMtmsr(msr & ~0x8000);
	return(msr);
}

/* cacheInitForTarget():
	Enable instruction cache only...
*/
void
cacheInitForTarget()
{
	asm(".equ	r3,3");
	asm(".equ	IC_CST,560");

	asm("	mfspr	r3,IC_CST");	
	asm("	lis		r3,0x0200");
	asm("	mtspr	IC_CST,r3");
	asm("	isync");
}
