/*
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:	Ed Sutter
 * email:	esutter@lucent.com
 * phone:	908-582-2351
 * 
 */
	.file "reset.s"

#include "config.h"
#include "arch_ppc.h"
#include "masks860.h"
#include "warmstart.h"

	.globl	coldstart
	.globl	endlessloop
	.globl	warmstart
	.globl  SPInit
	.extern	start
	.extern MonStack

/* Chip Select Assignments:
 */
#define	CS0_BR_VAL		0xff000801	/* 16MByte boot flash */
#define	CS0_OR_VAL		0xff000150

#define	CS1_BR_VAL		0x000000c1	/* 32Mbyte SDRAM */
#define	CS1_OR_VAL		0xfe000f00

#define	CS3_BR_VAL		0x30000801	/* 32K CPLD */
#define	CS3_OR_VAL		0xffff81f4

#define	CS4_BR_VAL		0x30008001	/* FPGA */
#define	CS4_OR_VAL		0xffff8150

#define	CS5_BR_VAL		0x30010401	/* 32K EEPROM */
#define	CS5_OR_VAL		0xffff80f4

#define	MAMR_VAL		0x31164000
#define	MBMR_VAL1		0x31964000
#define	MBMR_VAL2		0x31904000
#define	MAR_VAL			0x00000088
#define	MCR_VAL			0x80802235

#define SYPCR_VAL		0xffffff89
#define SIUMCR_VAL		0x00000400

#define SCCRK_VAL1		0x55ccaa33	/* Unlock SCCR register */
#define SCCR_VAL		0x03020800
#define SCCRK_VAL2		0x00000000	/* Lock SCCR register */

#define PLPRCRK_VAL1	0x55ccaa33	/* Unlock PLPRCR register */
#define PLPRCR_VAL		0x000a4000
#define PLPRCRK_VAL2	0x00000000	/* Lock PLPRCR register */

#define IMMR_BASE		0xa0000000

	.text

/* The first 0x100 bytes are up for grabs, so fill the space with an
 * ID string, the MONCOMPTR and space for FLASH-stored MAC address...
 */
	.string "@(#)MicroMonitor"

	.balign 0x10

#include "moncomptr.S"
#include "etheraddr.S"

	.balign		0x100

coldstart:
	li	r3, INITIALIZE

warmstart:
	/* Initialize IMMR (Internal Memory Map Reg):
	 * Only the upper 16 bits of this register are writeable.
	 *
	 * NOTE THAT R4 IS USED THROUGHOUT THIS RESET CODE for
	 * IMMR-relative addressing, so don't use it anywhere else prior
	 * to jumping into C-code.
	 */
	lis		r4, (IMMR_BASE)@h
	ori		r4, r4, (IMMR_BASE)@l
	mtspr	IMMR, r4

	/* SYPCR: System Protection Ctrl
	 */
	lis		r5,(SYPCR_VAL)@h
	ori		r5,r5,(SYPCR_VAL)@l
	stw		r5,SYPCR_BASE(r4)

	/* SIUMCR: SIU Module Config Reg
	 */
    lis		r5,(SIUMCR_VAL)@h
    ori		r5,r5,(SIUMCR_VAL)@l
    stw		r5,SIUMCR_BASE(r4)

	lis 	r5,(SCCRK_VAL1)@h
	ori		r5,r5,(SCCRK_VAL1)@l
	stw		r5,SCCRK_BASE(r4)  

	lis 	r5,(SCCR_VAL)@h
	ori		r5,r5,(SCCR_VAL)@l
	stw		r5,SCCR_BASE(r4)  

	lis 	r5,(SCCRK_VAL2)@h
	ori		r5,r5,(SCCRK_VAL2)@l
	stw		r5,SCCRK_BASE(r4)  

	lis 	r5,(PLPRCRK_VAL1)@h
	ori		r5,r5,(PLPRCRK_VAL1)@l
	stw		r5,PLPRCRK_BASE(r4)  

	lis 	r5,(PLPRCR_VAL)@h
	ori		r5,r5,(PLPRCR_VAL)@l
	stw		r5,PLPRCR_BASE(r4)  

	lis 	r5,(PLPRCRK_VAL2)@h
	ori		r5,r5,(PLPRCRK_VAL2)@l
	stw		r5,PLPRCRK_BASE(r4)  

	lis     r7,(addr_adjust)@h			/* R7 = address of addr_adjust */
	ori     r7,r7,(addr_adjust)@l		/* relative to 0xff000000 */
	mtlr	r7

	/*	CS0 = Boot FLASH
	 */
	lis		r5,(CS0_BR_VAL)@h
	ori		r5,r5,(CS0_BR_VAL)@l
	lis		r6,(CS0_OR_VAL)@h
	ori		r6,r6,(CS0_OR_VAL)@l
	stw		r5,BR0_BASE(r4)	
	blr									/* jump to flash mem address */
addr_adjust:
	stw		r6,OR0_BASE(r4)

	/* Init UPMB
	 * Transfer the content of the UpmTable (below) to the memory
	 * controller using the MDR (memory data register) and MCR (memory
	 * command register).  This particular transfer is to UPMB  and
	 * must be done prior to configuring the chip-select regs
	 * dedicated to DRAM.
	 */
UPMInit:
	lis		r6, (UpmTable)@h			/* point R6 to parameter table */
	ori		r6, r6, (UpmTable)@l

	lis		r7, (UpmTableEnd)@h			/* point R7 to end parameter table */
	ori		r7, r7, (UpmTableEnd)@l

	sub		r5, r7, r6					/* set R5 to # of words in param tbl */
	srawi	r5, r5, 2					/* /4 */
	lis		r9, 0x0080					/* Command: OP=Write, UPMB, MAD=0 */

UpmWriteLoop:
	lwz		r10, 0(r6)					/* get data from table */
	stw		r10, MDR_BASE(r4)			/* store the data to MD register */

	stw		r9, MCR_BASE(r4)			/* issue command to MCR register */

	addi	r6,r6,4						/* next entry in the table */
	addi	r9,r9,1						/* next MAD address */
	addi	r5,r5,-1					/* decrement counter */
	cmpwi	r5,0						/* done yet ? */
	bne		UpmWriteLoop

	/* Set DRAM refresh rate
	 */
	li		r5, MPTPR_PTP_DIV2
	sth		r5, MPTPR_BASE(r4)

	lis		r5, (MBMR_VAL1)@h 
	ori		r5, r5, (MBMR_VAL1)@l
	stw		r5, MBMR_BASE(r4)

	/* Chip Select Initialization:
	 */ 

	/*	CS1 = DRAM
	 */
	lis		r5,(CS1_OR_VAL)@h
	ori		r5,r5,(CS1_OR_VAL)@l
	stw		r5,OR1_BASE(r4)
	lis		r5,(CS1_BR_VAL)@h
	ori		r5,r5,(CS1_BR_VAL)@l
	stw		r5,BR1_BASE(r4)

	/*	CS3 = CPLD
	 */
	lis		r5,(CS3_OR_VAL)@h
	ori		r5,r5,(CS3_OR_VAL)@l
	stw		r5,OR3_BASE(r4)
	lis		r5,(CS3_BR_VAL)@h
	ori		r5,r5,(CS3_BR_VAL)@l
	stw		r5,BR3_BASE(r4)

	/*	CS4 = 
	 */
	lis		r5,(CS4_OR_VAL)@h
	ori		r5,r5,(CS4_OR_VAL)@l
	stw		r5,OR4_BASE(r4)
	lis		r5,(CS4_BR_VAL)@h
	ori		r5,r5,(CS4_BR_VAL)@l
	stw		r5,BR4_BASE(r4)

	/*	CS5 = EEPROM
	 */
	lis		r5,(CS5_OR_VAL)@h
	ori		r5,r5,(CS5_OR_VAL)@l
	stw		r5,OR5_BASE(r4)
	lis		r5,(CS5_BR_VAL)@h
	ori		r5,r5,(CS5_BR_VAL)@l
	stw		r5,BR5_BASE(r4)

	/* turn LEDs on:
	 */
	lis		r5,0x30000000@h
	lis		r6,0x0
	stw		r6,0(r5)

	/* Delay:
	 */
	lis		r5,0x00040000@h
	ori		r5,r5,0x00040000@l
dly1:
	addi	r5,r5,-1
	cmpwi	r5,0
	bgt+	dly1

	lis		r5,(MAR_VAL)@h
	ori		r5,r5,(MAR_VAL)@l
	stw		r5,MAR_BASE(r4)

	lis		r5,(MCR_VAL)@h
	ori		r5,r5,(MCR_VAL)@l
	stw		r5,MCR_BASE(r4)

	lis		r5,0x00040000@h
	ori		r5,r5,0x00040000@l
dly2:
	addi	r5,r5,-1
	cmpwi	r5,0
	bgt+	dly2
	
	lis		r5,(MBMR_VAL2)@h
	ori		r5,r5,(MBMR_VAL2)@l
	stw		r5,MBMR_BASE(r4)
	
	/* turn LEDs off:
	 */
	lis		r5,0x30000000@h
	lis		r6,0xffffffff@h
	ori		r6,r6,0xffffffff@l
	stw		r6,0(r5)

SPInit:
	/* initialize the stack pointer, and be assured that it is mod16 */
	lis		sp, (MonStack+(MONSTACKSIZE-4))@h
	ori		sp, sp, (MonStack+(MONSTACKSIZE-4))@l
	addis	r5, r0,	0xffff
	ori		r5,	r5, 0xfff8
	and		r1,	r1, r5

GoToC:
	lis		r7,(start)@h
	ori		r7,r7,(start)@l
	mtlr	r7							/* move C entry point to LR */
	blr									/* jump to the C entry point */


UpmTable:

# Single Read
	.long	0x0f03c004, 0x0f0ec004, 0x00bdc004, 0x0fffc000
	.long	0xffffc004, 0x1ff74004, 0xffffcc05, 0xffffcc05

# Burst Read
	.long	0x0fffcc24, 0x0fffcc04, 0x08ffcc04, 0x00ffcc04
	.long	0x00ffcc08, 0x0cffcc44, 0x00ffec0c, 0x03ffec00
	.long	0x00ffec44, 0x00ffcc08, 0x0cffcc44, 0x00ffec04
	.long	0x00ffec00, 0x3fffec47, 0xffffffff, 0xffffffff

	# Single Write
	.long	0x0f03cc04, 0x0f0fc804, 0x0fafc004, 0x0ebe8400
	.long	0x00fd4c04, 0x0fffcc04, 0x1fffcc04, 0xffffcc05

	# Burst Write
	.long	0x0fafcc24, 0x0fafcc04, 0x08afcc00, 0x07afcc4c
	.long	0x08afcc00, 0x07afcc4c, 0x08afcc00, 0x07afcc4c
	.long	0x08afcc00, 0x37afcc47, 0xffffffff, 0xffffffff
	.long	0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff

	# Refresh
	.long	0xefffcc04, 0x0ffacc04, 0x0ff5cc04, 0x1fffcc04
	.long	0xfffbcc05, 0xeff7cc04, 0x0fffcc84, 0x0ff5cc04
	.long	0x0ffa8c84, 0x0ff54c04, 0x0fffc004, 0x0fffc00d

	# Exception
	.long	0x0ffa8004, 0x1ff54004, 0xffffcc04, 0xffffcc05

UpmTableEnd:
