/* cpuio.h:
 *	Helpful macros for register access. Inclusion of all sh7720 specific
 *	header files.
 *
 *	General notice:
 *	This code is part of a boot-monitor package developed as a generic base
 *	platform for embedded system designs.  As such, it is likely to be
 *	distributed to various projects beyond the control of the original
 *	author.  Please notify the author of any enhancements made or bugs found
 *	so that all may benefit from the changes.  In addition, notification back
 *	to the author will allow the new user to pick up changes that may have
 *	been made by other users after this version of the code was distributed.
 *
 *	Author:	Torsten Tetz, Bosch Security Systems
 *	        Mark Jonas, Bosch Security Systems, mark.jonas@de.bosch.com
 *	        Joerg Tuermer, Bosch Security Systems, joerg.tuermer@de.bosch.com
 */
#ifndef __CPUIO_H
#define __CPUIO_H

/* SH7720 peripherals. */
#include "sh7720/adc.h"
#include "sh7720/afeif.h"
#include "sh7720/bsc.h"
#include "sh7720/cache.h"
#include "sh7720/cmt.h"
#include "sh7720/cpg.h"
#include "sh7720/dac.h"
#include "sh7720/dma.h"
#include "sh7720/exception.h"
#include "sh7720/h-udi.h"
#include "sh7720/iic.h"
#include "sh7720/intcntrl.h"
#include "sh7720/irda.h"
#include "sh7720/lcdc.h"
#include "sh7720/lowpower.h"
#include "sh7720/mmc.h"
#include "sh7720/mmu.h"
#include "sh7720/pcc.h"
#include "sh7720/pfc.h"
#include "sh7720/port.h"
#include "sh7720/rtc.h"
#include "sh7720/scif.h"
#include "sh7720/sim.h"
#include "sh7720/siof.h"
#include "sh7720/ssl.h"
#include "sh7720/tmu.h"
#include "sh7720/tpu.h"
#include "sh7720/ubc.h"
#include "sh7720/usbf.h"
#include "sh7720/usbh.h"
#include "sh7720/usbpmc.h"
#include "sh7720/wdt.h"

/* CPU register read and write function prototypes needed in except_sh3.c. */
unsigned long getsr(void);
void putsr(unsigned long sr);
void putvbr(unsigned long vbr);

/* Macros for writing registers. */
#define ctrl_outb(val, adr) *adr = (val)
#define ctrl_outw(val, adr) *adr = (val)
#define ctrl_outl(val, adr) *adr = (val)

/* Macros for reading registers. */
#define ctrl_inb(adr) (*(adr))
#define ctrl_inw(adr) (*(adr))
#define ctrl_inl(adr) (*(adr))

/* Macros for setting and clearing register bits. */
#define ctrl_bit_set_outb(mask, reg)     ctrl_outb(ctrl_inb(reg) | mask, reg)
#define ctrl_bit_set_outw(mask, reg)     ctrl_outw(ctrl_inw(reg) | mask, reg)
#define ctrl_bit_set_outl(mask, reg)     ctrl_outl(ctrl_inl(reg) | mask, reg)
#define ctrl_bit_clear_outb(mask, reg)   ctrl_outb(ctrl_inb(reg) & ~mask, reg)
#define ctrl_bit_clear_outw(mask, reg)   ctrl_outw(ctrl_inw(reg) & ~mask, reg)
#define ctrl_bit_clear_outl(mask, reg)   ctrl_outl(ctrl_inl(reg) & ~mask, reg)

#endif	/* __CPUIO_H */

